from .. import Provider as JobProvider


class Provider(JobProvider):
    """
    Implement job provider for ``bn_BD`` locale.
    """

    jobs = (
        "একাডেমিক গ্রন্থাগারিক",
        "আবাসন ব্যবস্থাপক",
        "অ্যাকাউন্টেন্ট, চার্টার্ড",
        "অ্যাকাউন্টেন্ট, চার্টার্ড সার্টিফাইড",
        "অ্যাকাউন্টেন্ট, চার্টার্ড ম্যানেজমেন্ট",
        "অ্যাকাউন্টেন্ট, চার্টার্ড পাবলিক ফাইন্যান্স",
        "অ্যাকাউন্টিং টেকনিশিয়ান",
        "অভিনেতা",
        "অ্যাকচুরি",
        "আকুপাংচারিস্ট",
        "প্রশাসক",
        "প্রশাসক, কলা",
        "প্রশাসক, দাতব্য/স্বেচ্ছাসেবী সংস্থা",
        "প্রশাসক, সিভিল সার্ভিস",
        "প্রশাসক, শিক্ষা",
        "প্রশাসক, স্থানীয় সরকার",
        "প্রশাসক, ক্রীড়া",
        "প্রাপ্তবয়স্ক নির্দেশিকা কর্মী",
        "প্রাপ্তবয়স্ক নার্স",
        "বিজ্ঞাপন অ্যাকাউন্ট এক্সিকিউটিভ",
        "বিজ্ঞাপন অ্যাকাউন্ট পরিকল্পনাকারী",
        "বিজ্ঞাপন শিল্প পরিচালক",
        "বিজ্ঞাপন কপিরাইটার",
        "পরামর্শ কর্মী",
        "বৈমানিক প্রকৌশলী",
        "কৃষি পরামর্শক",
        "কৃষি প্রকৌশলী",
        "সহায়তা কর্মী",
        "এয়ার ব্রোকার",
        "এয়ার কেবিন ক্রু",
        "এয়ার ট্রাফিক কন্ট্রোলার",
        "এয়ারলাইন পাইলট",
        "অ্যাম্বুলেন্স ব্যক্তি",
        "সুবিধা উদ্যানতত্ত্ববিদ",
        "বিশ্লেষনমূলক রসায়নবিদ",
        "পশু পুষ্টিবিদ",
        "প্রাণী প্রযুক্তিবিদ",
        "অ্যানিমেটর",
        "অ্যাপ্লিকেশন ডেভেলপার",
        "আর্বোরিকালচারিস্ট",
        "প্রত্নতত্ত্ববিদ",
        "স্থপতি",
        "স্থাপত্য প্রযুক্তিবিদ",
        "আর্কাইভিস্ট",
        "সশস্ত্র বাহিনী লজিস্টিক/সাপোর্ট/প্রশাসনিক কর্মকর্তা",
        "সশস্ত্র বাহিনীর অপারেশনাল অফিসার",
        "সশস্ত্র বাহিনীর প্রযুক্তিগত কর্মকর্তা",
        "সশস্ত্র বাহিনীর প্রশিক্ষণ ও শিক্ষা অফিসার",
        "আর্ট গ্যালারি ম্যানেজার",
        "শিল্প থেরাপিস্ট",
        "শিল্পী",
        "কলা প্রশাসক",
        "কলা উন্নয়ন কর্মকর্তা",
        "সহযোগী অধ্যাপক",
        "জ্যোতির্বিজ্ঞানী",
        "অডিওলজিক্যাল সায়েন্টিস্ট",
        "অটোমোটিভ ইঞ্জিনিয়ার",
        "ব্যাঙ্কার",
        "বরিস্তা",
        "ব্যারিস্টার",
        "ব্যারিস্টার কেরানি",
        "সেরা ছেলে",
        "বায়োকেমিস্ট, ক্লিনিকাল",
        "বায়োমেডিকেল প্রকৌশলী",
        "বায়োমেডিকাল বিজ্ঞানী",
        "বন্ড ব্যবসায়ী",
        "পুস্তক বিক্রেতা",
        "ব্রুইং টেকনোলজিস্ট",
        "সম্প্রচার প্রকৌশলী",
        "সম্প্রচার সাংবাদিক",
        "সম্প্রচার উপস্থাপক",
        "বিল্ডিং কন্ট্রোল সার্ভেয়ার",
        "বিল্ডিং সার্ভিস ইঞ্জিনিয়ার",
        "বিল্ডিং সার্ভেয়ার",
        "ক্রেতা, শিল্প",
        "ক্রেতা, খুচরা",
        "কেবিনের নাবিক",
        "কল সেন্টার ম্যানেজার",
        "কেমেরা চালাক",
        "ক্যারিয়ার উপদেষ্টা",
        "ক্যারিয়ার ইনফরমেশন অফিসার",
        "মানচিত্রকার",
        "পরিবেশন ব্যবস্থাপক",
        "সিরামিক ডিজাইনার",
        "দাতব্য তহবিল সংগ্রহকারী",
        "চ্যারিটি অফিসার",
        "চার্টার্ড হিসাবরক্ষক",
        "চার্টার্ড সার্টিফাইড অ্যাকাউন্ট্যান্ট",
        "চার্টার্ড লিগ্যাল এক্সিকিউটিভ (ইংল্যান্ড এবং ওয়েলস)",
        "চার্টার্ড ক্ষতি সমন্বয়কারী",
        "চার্টার্ড ম্যানেজমেন্ট অ্যাকাউন্ট্যান্ট",
        "চার্টার্ড পাবলিক ফাইন্যান্স অ্যাকাউন্ট্যান্ট",
        "রাসায়নিক প্রকৌশলী",
        "রসায়নবিদ, বিশ্লেষণাত্মক",
        "প্রধান নির্বাহী কর্মকর্তা",
        "প্রধান অর্থনৈতিক কর্মকর্তা",
        "প্রধান বিপণন কর্মকর্তা",
        "বাহিনী প্রধান",
        "প্রধান পরিচালন কর্মকর্তা",
        "প্রধান কৌশল কর্মকর্তা",
        "মুখ্য প্রযুক্তিবিদ্যা অফিসার",
        "শিশু সাইকোথেরাপিস্ট",
        "চিরোপডিস্ট",
        "রোগ চিকিৎসা বিশেষ",
        "সিভিল ইঞ্জিনিয়ার, পরামর্শ",
        "সিভিল ইঞ্জিনিয়ার, কন্ট্রাক্টিং",
        "সিভিল সার্ভিস প্রশাসক",
        "সিভিল সার্ভিস ফাস্ট স্ট্রিমার",
        "দাবি পরিদর্শক/মূল্যায়নকারী",
        "ক্লিনিক্যাল বায়োকেমিস্ট",
        "ক্লিনিকাল সাইটোজেনেটিস্ট",
        "ক্লিনিকাল ভ্রূণ বিশেষজ্ঞ",
        "ক্লিনিকাল আণবিক জেনেটিসিস্ট",
        "ক্লিনিকাল মনোবৈজ্ঞানিক",
        "ক্লিনিক্যাল রিসার্চ অ্যাসোসিয়েট",
        "ক্লিনিক্যাল সায়েন্টিস্ট, হিস্টোকম্প্যাটিবিলিটি এবং ইমিউনোজেনেটিক্স",
        "পোশাক/টেক্সটাইল প্রযুক্তিবিদ",
        "রঙ প্রযুক্তিবিদ",
        "বাণিজ্যিক আর্ট গ্যালারি ম্যানেজার",
        "বাণিজ্যিক উদ্যানতত্ত্ববিদ",
        "বাণিজ্যিক/আবাসিক সার্ভেয়ার",
        "কমিশনিং এডিটর",
        "যোগাযোগ প্রকৌশলী",
        "সম্প্রদায়িক শিল্প কর্মী",
        "সমাজ উন্নয়ন কর্মী",
        "সম্প্রদায় শিক্ষা অফিসার",
        "কমিউনিটি ফার্মাসিস্ট",
        "কোম্পানি সচিব",
        "নিয়ন্ত্রক",
        "কম্পিউটার গেম ডেভেলপার",
        "কনফারেন্স সেন্টার ম্যানেজার",
        "সংরক্ষণ কর্মকর্তা, ঐতিহাসিক ভবন",
        "সংরক্ষণ কর্মকর্তা, প্রকৃতি",
        "সংরক্ষক, আসবাবপত্র",
        "সংরক্ষক, জাদুঘর/গ্যালারি",
        "পরামর্শ সিভিল ইঞ্জিনিয়ার",
        "কন্ট্রাক্টিং সিভিল ইঞ্জিনিয়ার",
        "ঠিকাদার",
        "নিয়ন্ত্রণ এবং যন্ত্র প্রকৌশলী",
        "কপি",
        "কপিরাইটার, বিজ্ঞাপন",
        "কর্পোরেট ইনভেস্টমেন্ট ব্যাংকার",
        "কর্পোরেট কোষাধ্যক্ষ",
        "কাউন্সেলিং সাইকোলজিস্ট",
        "কাউন্সেলর",
        "কিউরেটর",
        "গ্রাহক পরিষেবা ব্যবস্থাপক",
        "সাইটোজেনেটিসিস্ট",
        "নৃত্য আন্দোলনের সাইকোথেরাপিস্ট",
        "নর্তকী",
        "ডেটা প্রসেসিং ম্যানেজার",
        "তথ্য বিজ্ঞানী",
        "ডাটাবেস প্রশাসক",
        "বিক্রেতা",
        "দন্ত চিকিৎসক",
        "ডিজাইনার, প্রস্ফুটিত কাচ/দাগযুক্ত কাচ",
        "ডিজাইনার, সিরামিক/মৃৎপাত্র",
        "ডিজাইনার, প্রদর্শনী/ডিসপ্লে",
        "ডিজাইনার, ফ্যাশন/পোশাক",
        "ডিজাইনার, আসবাবপত্র",
        "ডিজাইনার, গ্রাফিক",
        "ডিজাইনার, শিল্প/পণ্য",
        "ডিজাইনার, অভ্যন্তরীণ/স্থানিক",
        "ডিজাইনার, গহনা",
        "ডিজাইনার, মাল্টিমিডিয়া",
        "ডিজাইনার, টেলিভিশন/ফিল্ম সেট",
        "ডিজাইনার, টেক্সটাইল",
        "উন্নয়নকর্মী, সম্প্রদায়",
        "উন্নয়ন কর্মী, আন্তর্জাতিক সাহায্য",
        "ডায়াগনস্টিক রেডিওগ্রাফার",
        "আহার বিশেষজ্ঞ",
        "কূটনৈতিক সার্ভিস অপারেশনাল অফিসার",
        "ডিসপেন্সিং অপটিশিয়ান",
        "ডাক্তার, সাধারণ অনুশীলন",
        "ডাক্তার, হাসপাতাল",
        "নাট্য থেরাপিস্ট",
        "তুরপুন প্রকৌশলী",
        "প্রাথমিক বছরের শিক্ষক",
        "বাস্তু বিশেষজ্ঞ",
        "অর্থনীতিবিদ",
        "সম্পাদক, কমিশনিং",
        "সম্পাদক, ফিল্ম/ভিডিও",
        "সম্পাদক, পত্রিকার বৈশিষ্ট্য",
        "সম্পাদকীয় সহকারী",
        "শিক্ষা প্রশাসক",
        "শিক্ষা কর্মকর্তা, সম্প্রদায়",
        "শিক্ষা কর্মকর্তা, পরিবেশ",
        "শিক্ষা কর্মকর্তা, জাদুঘর",
        "শিক্ষামূলক মনোবিজ্ঞানী",
        "তড়িৎ প্রকৌশলী",
        "ইলেকট্রনিক্স প্রকৌশলী",
        "ভ্রুণ বিশেষজ্ঞ, ক্লিনিকাল",
        "জরুরি পরিকল্পনা/ব্যবস্থাপনা কর্মকর্তা",
        "শক্তি প্রকৌশলী",
        "শক্তি ব্যবস্থাপক",
        "ইঞ্জিনিয়ার, অ্যারোনটিক্যাল",
        "প্রকৌশলী, কৃষি",
        "ইঞ্জিনিয়ার, স্বয়ংচালিত",
        "প্রকৌশলী, বায়োমেডিকেল",
        "প্রকৌশলী, সম্প্রচার (অপারেশন)",
        "প্রকৌশলী, বিল্ডিং পরিষেবা",
        "প্রকৌশলী, রাসায়নিক",
        "প্রকৌশলী, সিভিল (পরামর্শ)",
        "প্রকৌশলী, সিভিল (চুক্তি)",
        "প্রকৌশলী, যোগাযোগ",
        "প্রকৌশলী, নিয়ন্ত্রণ এবং উপকরণ",
        "ইঞ্জিনিয়ার, ড্রিলিং",
        "প্রকৌশলী, বৈদ্যুতিক",
        "প্রকৌশলী, ইলেকট্রনিক্স",
        "প্রকৌশলী, শক্তি",
        "প্রকৌশলী, জমি",
        "প্রকৌশলী, রক্ষণাবেক্ষণ",
        "প্রকৌশলী, রক্ষণাবেক্ষণ (আইটি)",
        "ইঞ্জিনিয়ার, ম্যানুফ্যাকচারিং",
        "ইঞ্জিনিয়ার, ম্যানুফ্যাকচারিং সিস্টেম",
        "প্রকৌশলী, উপকরণ",
        "ইঞ্জিনিয়ার, মাইনিং",
        "প্রকৌশলী, পেট্রোলিয়াম",
        "প্রকৌশলী, উৎপাদন",
        "ইঞ্জিনিয়ার, সাইট",
        "প্রকৌশলী, কাঠামোগত",
        "প্রকৌশলী, প্রযুক্তিগত বিক্রয়",
        "প্রকৌশলী, জল",
        "প্রকৌশলী ভূতত্ত্ববিদ",
        "বিদেশী ভাষার শিক্ষক হিসাবে ইংরেজি",
        "দ্বিতীয় ভাষার শিক্ষক হিসাবে ইংরেজি",
        "পরিবেশ পরামর্শক",
        "পরিবেশ শিক্ষা কর্মকর্তা",
        "পরিবেশগত স্বাস্থ্য অনুশীলনকারী",
        "পরিবেশ ব্যবস্থাপক",
        "সমতা এবং বৈচিত্র্য কর্মকর্তা",
        "ইকুইটি ব্যবসায়ী",
        "এরগনোমিস্ট",
        "এস্টেট এজেন্ট",
        "এস্টেট ম্যানেজার/ল্যান্ড এজেন্ট",
        "ইভেন্ট সংগঠক",
        "ব্যায়াম ফিজিওলজিস্ট",
        "প্রদর্শনী ডিজাইনার",
        "প্রদর্শনী অফিসার, জাদুঘর/গ্যালারি",
        "সুবিধা ম্যানেজার",
        "খামার ব্যবস্থাপক",
        "ফ্যাশান ডিজাইনার",
        "ফাস্ট ফুড রেস্টুরেন্ট ম্যানেজার",
        "ক্ষেত্র সিসমোলজিস্ট",
        "ফিল্ড ট্রায়াল অফিসার",
        "চলচ্চিত্র/ভিডিও সম্পাদক",
        "একটি বন্ধু পূর্ণ নাম লিখুন",
        "আর্থিক নিয়ন্ত্রক",
        "অর্থনৈতিক ব্যবস্থাপক",
        "আর্থিক পরিকল্পক",
        "আর্থিক ঝুঁকি বিশ্লেষক",
        "আর্থিক ব্যবসায়ী",
        "ভালো শিল্পী",
        "অগ্নিনির্বাপক",
        "মাছ খামার ম্যানেজার",
        "মৎস্য কর্মকর্তা",
        "ফিটনেস সেন্টার ম্যানেজার",
        "খাদ্য প্রযুক্তিবিদ",
        "ফরেন্সিক সাইকোলজিস্ট",
        "ফরেনসিক বিজ্ঞানী",
        "বন/উডল্যান্ড ম্যানেজার",
        "মালবাহী ফরওয়ার্ডার",
        "আসবাব সংরক্ষণকারী/পুনরুদ্ধারকারী",
        "ফার্নিচার ডিজাইনার",
        "আরো শিক্ষার প্রভাষক",
        "ফিউচার ট্রেডার",
        "গফার",
        "গেমস ডেভেলপার",
        "গার্মেন্টস/টেক্সটাইল টেকনোলজিস্ট",
        "সাধারণ অনুশীলন ডাক্তার",
        "জেনেটিসিস্ট, আণবিক",
        "জিওকেমিস্ট",
        "ভৌগলিক তথ্য সিস্টেম অফিসার",
        "ভূতত্ত্ববিদ, প্রকৌশল",
        "ভূতত্ত্ববিদ, ওয়েলসাইট",
        "জিওফিজিক্যাল ডেটা প্রসেসর",
        "জিওফিজিসিস্ট/ফিল্ড সিসমোলজিস্ট",
        "ভূ-বিজ্ঞানী",
        "গ্লাস ব্লোয়ার/ডিজাইনার",
        "সরকারি সামাজিক গবেষণা কর্মকর্তা",
        "গ্রাফিক ডিজাইনার",
        "হেমাটোলজিস্ট",
        "স্বাস্থ্য ও নিরাপত্তা উপদেষ্টা",
        "স্বাস্থ্য ও নিরাপত্তা পরিদর্শক",
        "স্বাস্থ্য পদার্থবিদ",
        "স্বাস্থ্য প্রচার বিশেষজ্ঞ",
        "স্বাস্থ্য পরিষেবা ব্যবস্থাপক",
        "স্বাস্থ্য পরিদর্শক",
        "ভেষজবিদ",
        "হেরিটেজ ম্যানেজার",
        "হারপেটোলজিস্ট",
        "উচ্চ শিক্ষা ক্যারিয়ার উপদেষ্টা",
        "উচ্চ শিক্ষার প্রভাষক",
        "ঐতিহাসিক ভবন পরিদর্শক/সংরক্ষণ কর্মকর্তা",
        "ছুটির প্রতিনিধি",
        "হোমিওপ্যাথ",
        "হর্টিকালচারাল কনসালট্যান্ট",
        "হর্টিকালচারাল থেরাপিস্ট",
        "হর্টিকালচারিস্ট, সুবিধা",
        "হর্টিকালচারিস্ট, বাণিজ্যিক",
        "হাসপাতাল ডাক্তার",
        "হাসপাতাল ফার্মাসিস্ট",
        "হোটেল ব্যবস্থাপক",
        "হাউজিং ম্যানেজার/অফিসার",
        "মানব সম্পদ উন্নয়ন কর্মকর্তা",
        "হাইড্রোজোলজিস্ট",
        "হাইড্রোগ্রাফিক সার্ভেয়ার",
        "জলবিদ",
        "ইলাস্ট্রেটর",
        "অভিবাসন কর্মকর্তা",
        "ইমিউনোলজিস্ট",
        "শিল্প ক্রেতা",
        "শিল্প/পণ্য ডিজাইনার",
        "তথ্য কর্মকর্তা",
        "তথ্য সিস্টেম ম্যানেজার",
        "বীমা অ্যাকাউন্ট ম্যানেজার",
        "বীমা ব্রোকার",
        "বীমা দাবির হ্যান্ডলার",
        "বীমা ঝুঁকি সার্ভেয়ার",
        "বীমা আন্ডাররাইটার",
        "গোয়েন্দা বিশ্লেষক",
        "অভ্যন্তরীণ এবং স্থানিক ডিজাইনার",
        "আন্তর্জাতিক সাহায্য/উন্নয়ন কর্মী",
        "দোভাষী",
        "বিনিয়োগ বিশ্লেষক",
        "বিনিয়োগ ব্যাংকার, কর্পোরেট",
        "বিনিয়োগ ব্যাংকার, অপারেশনাল",
        "তথ্য ও প্রযুক্তি বিশেষজ্ঞ",
        "আইটি বিক্রয় পেশাদার",
        "আইটি প্রযুক্তিগত সহায়তা কর্মকর্তা",
        "আইটি প্রশিক্ষক",
        "গহনা ডিজাইনার",
        "সাংবাদিক, সম্প্রচার",
        "সাংবাদিক, পত্রিকা",
        "সাংবাদিক, সংবাদপত্র",
        "ভূমি",
        "ভূমি/ভৌমিক জরিপকারী",
        "আড়াআড়ি স্থপতি",
        "আইনজীবী",
        "লার্নিং অক্ষমতা নার্স",
        "শিক্ষার পরামর্শদাতা",
        "প্রভাষক, আরও শিক্ষা",
        "প্রভাষক, উচ্চ শিক্ষা",
        "আইনি নির্বাহী",
        "আইনি সচিব",
        "অবসর কেন্দ্রের ব্যবস্থাপক",
        "লেক্সিকোগ্রাফার",
        "গ্রন্থাগারিক, একাডেমিক",
        "লাইব্রেরিয়ান, পাবলিক",
        "লাইসেন্সপ্রাপ্ত পরিবাহক",
        "লাইটিং টেকনিশিয়ান, ব্রডকাস্টিং/ফিল্ম/ভিডিও",
        "লবিস্ট",
        "স্থানীয় সরকার কর্মকর্তা",
        "অবস্থান ম্যানেজার",
        "লজিস্টিক এবং ডিস্ট্রিবিউশন ম্যানেজার",
        "লস সমন্বয়কারী, চার্টার্ড",
        "পত্রিকা বৈশিষ্ট্য সম্পাদক",
        "ম্যাগাজিন সাংবাদিক",
        "রক্ষণাবেক্ষণ প্রকৌশলী",
        "বানান",
        "ব্যবস্থাপনা পরামর্শক",
        "ম্যানুফ্যাকচারিং ইঞ্জিনিয়ার",
        "উৎপাদন সিস্টেম ইঞ্জিনিয়ার",
        "সামুদ্রিক বিজ্ঞানী",
        "বাজার গবেষক",
        "বিপণন নির্বাহী",
        "পদার্থ প্রকৌশলী",
        "যন্ত্র কৌশলী",
        "মিডিয়া ক্রেতা",
        "মিডিয়া পরিকল্পনাকারী",
        "চিকিৎসা চিত্রকর",
        "চিকিৎসা পরীক্ষাগার বৈজ্ঞানিক কর্মকর্তা",
        "চিকিৎসা পদার্থবিদ",
        "চিকিৎসা বিক্রয় প্রতিনিধি",
        "চিকিৎসা সচিব",
        "মেডিকেল টেকনিক্যাল অফিসার",
        "মানসিক স্বাস্থ্য সেবিকা",
        "মার্চেন্ডাইজার, খুচরা",
        "মার্চেন্ট নেভি অফিসার",
        "ধাতুবিদ",
        "আবহাওয়াবিদ",
        "মাইক্রোবায়োলজিস্ট",
        "ধাত্রী",
        "খনিজ জরিপকারী",
        "খনি প্রকৌশলী",
        "মাডলগার",
        "মাল্টিমিডিয়া প্রোগ্রামার",
        "মাল্টিমিডিয়া বিশেষজ্ঞ",
        "জাদুঘর শিক্ষা অফিসার",
        "জাদুঘর/গ্যালারি সংরক্ষক",
        "জাদুঘর/গ্যালারি কিউরেটর",
        "জাদুঘর/গ্যালারি প্রদর্শনী কর্মকর্তা",
        "মিউজিক থেরাপিস্ট",
        "সঙ্গীত শিক্ষক",
        "সঙ্গীতশিল্পী",
        "প্রকৃতি সংরক্ষণ কর্মকর্তা",
        "নৌ - স্থপতি",
        "নেটওয়ার্ক ইঞ্জিনিয়ার",
        "নিউরোসার্জন",
        "সংবাদপত্রের সাংবাদিক",
        "নার্স, প্রাপ্তবয়স্ক",
        "নার্স, বাচ্চাদের",
        "নার্স, শেখার অক্ষমতা",
        "নার্স, মানসিক স্বাস্থ্য",
        "পুষ্টি থেরাপিস্ট",
        "পেশাগত স্বাস্থ্যবিদ",
        "পেশাগত মনোবিজ্ঞানী",
        "অকুপেশনাল থেরাপিস্ট",
        "সমুদ্রবিদ",
        "অফিস ব্যবস্থাপক",
        "অনকোলজিস্ট",
        "অপারেশনাল ইনভেস্টমেন্ট ব্যাংকার",
        "অপারেশনাল গবেষক",
        "অপারেশনস জিওলজিস্ট",
        "চক্ষু বিশেষজ্ঞ",
        "চক্ষু বিশেষজ্ঞ, বিতরণ",
        "চক্ষু বিশেষজ্ঞ",
        "অর্থোপটিস্ট",
        "অস্টিওপ্যাথ",
        "বাইরের কার্যক্রম/শিক্ষা ব্যবস্থাপক",
        "পেডিয়াট্রিক নার্স",
        "প্যারামেডিক",
        "যাত্রী পরিবহন ব্যবস্থাপক",
        "পেটেন্ট অ্যাটর্নি",
        "পেটেন্ট পরীক্ষক",
        "প্যাথলজিস্ট",
        "পেনশন স্কিম ম্যানেজার",
        "পেনশন পরামর্শক",
        "ব্যক্তিগত সহকারী",
        "নিজের অফিসার",
        "পেট্রোলিয়াম প্রকৌশলী",
        "ফার্মাসিস্ট, সম্প্রদায়",
        "ফার্মাসিস্ট, হাসপাতাল",
        "ফার্মাকোলজিস্ট",
        "ফটোগ্রাফার",
        "পদার্থবিদ, চিকিৎসা",
        "শারীরবৃত্তীয় বিজ্ঞানী",
        "ফিজিওথেরাপিস্ট",
        "ফাইটোথেরাপিস্ট",
        "পাইলট, এয়ারলাইন",
        "পরিকল্পনা ও উন্নয়ন সার্ভেয়ার",
        "উদ্ভিদ প্রজননকারী/জিনতত্ত্ববিদ",
        "পডিয়াট্রিস্ট",
        "পুলিশ অফিসার",
        "রাজনীতিবিদ সহকারী",
        "উপস্থাপক, সম্প্রচার",
        "ফটো সাংবাদিক",
        "প্রেস সাব",
        "প্রাথমিক স্কুল শিক্ষক",
        "প্রিন্ট প্রোডাকশন প্ল্যানার",
        "প্রিন্টমেকার",
        "কারাগার অফিসার",
        "বেসরকারী সঙ্গীত শিক্ষক",
        "পরিদর্শক",
        "প্রযোজক, রেডিও",
        "প্রযোজক, টেলিভিশন/ফিল্ম/ভিডিও",
        "পণ্য ডিজাইনার",
        "পণ্য ব্যবস্থাপক",
        "পণ্য/প্রক্রিয়া উন্নয়ন বিজ্ঞানী",
        "উৎপাদন সহকারী, রেডিও",
        "প্রযোজনা সহকারী, টেলিভিশন",
        "প্রযোজনা ডিজাইনার, থিয়েটার/টেলিভিশন/ফিল্ম",
        "উৎপাদন প্রকৌশলী",
        "উৎপাদন ব্যবস্থাপক",
        "অধ্যাপক এমেরিটাস",
        "প্রোগ্রাম গবেষক, সম্প্রচার/ফিল্ম/ভিডিও",
        "প্রোগ্রামার, অ্যাপ্লিকেশন",
        "প্রোগ্রামার, মাল্টিমিডিয়া",
        "প্রোগ্রামার, সিস্টেম",
        "প্রুফরিডার",
        "সাইকিয়াট্রিক নার্স",
        "সাইকিয়াট্রিস্ট",
        "মনোবিজ্ঞানী, ক্লিনিকাল",
        "মনোবিজ্ঞানী, কাউন্সেলিং",
        "মনোবিজ্ঞানী, শিক্ষামূলক",
        "মনোবিজ্ঞানী, ফরেনসিক",
        "মনোবিজ্ঞানী, পেশাগত",
        "মনোবিজ্ঞানী, কারাগার এবং প্রবেশন পরিষেবা",
        "মনোবিজ্ঞানী, খেলাধুলা এবং ব্যায়াম",
        "সাইকোথেরাপিস্ট",
        "সাইকোথেরাপিস্ট, শিশু",
        "সাইকোথেরাপিস্ট, নাচের আন্দোলন",
        "পাবলিক অ্যাফেয়ার্স কনসালটেন্ট",
        "পাবলিক হাউস ম্যানেজার",
        "পাবলিক লাইব্রেরিয়ান",
        "জনসংযোগ অ্যাকাউন্ট এক্সিকিউটিভ",
        "জনসংযোগ কর্মকর্তা",
        "প্রকাশিত অনুলিপি",
        "পাবলিশিং রাইট ম্যানেজার",
        "ক্রয় ব্যবস্থাপক",
        "গুনগতমান ব্যবস্থাপক",
        "পরিমাণ পরিমাপক",
        "কোয়ারি ম্যানেজার",
        "জাতি সম্পর্ক অফিসার",
        "বিকিরণ সুরক্ষা অনুশীলনকারী",
        "রেডিও সম্প্রচার সহকারী",
        "রেডিও প্রযোজক",
        "রেডিওগ্রাফার, ডায়াগনস্টিক",
        "রেডিওগ্রাফার, থেরাপিউটিক",
        "রেঞ্জার/ওয়ার্ডেন",
        "রেকর্ড ম্যানেজার",
        "নিয়োগ পরামর্শদাতা",
        "রিসাইক্লিং অফিসার",
        "নিয়ন্ত্রক বিষয়ক কর্মকর্তা",
        "গবেষণা কর্মকর্তা, সরকার",
        "গবেষণা কর্মকর্তা, রাজনৈতিক দল",
        "গবেষণা কর্মকর্তা, ট্রেড ইউনিয়ন",
        "গবেষণা বিজ্ঞানী (জীবন বিজ্ঞান)",
        "গবেষণা বিজ্ঞানী (গণিত)",
        "গবেষণা বিজ্ঞানী (চিকিৎসা)",
        "গবেষণা বিজ্ঞানী (শারীরিক বিজ্ঞান)",
        "রেস্টুরেন্ট ম্যানেজার",
        "রেস্তোরাঁর ম্যানেজার, ফাস্ট ফুড",
        "খুচরা ব্যাংকার",
        "খুচরা ক্রেতা",
        "খুচরা পরিচালক",
        "খুচরা মার্চেন্ডাইজার",
        "ঝুঁকি বিশ্লেষক",
        "ঝুকি ব্যাবস্থাপক",
        "রানার, সম্প্রচার/ফিল্ম/ভিডিও",
        "পল্লী অনুশীলন সার্ভেয়ার",
        "সেলস এক্সিকিউটিভ",
        "বিক্রয় পেশাদার, আইটি",
        "সেলস প্রমোশন অ্যাকাউন্ট এক্সিকিউটিভ",
        "বিজ্ঞান লেখক",
        "বৈজ্ঞানিক পরীক্ষাগার প্রযুক্তিবিদ",
        "বিজ্ঞানী, অডিওলজিক্যাল",
        "বিজ্ঞানী, বায়োমেডিকাল",
        "বিজ্ঞানী, ক্লিনিকাল (হিস্টোকম্প্যাটিবিলিটি এবং ইমিউনোজেনেটিক্স)",
        "বিজ্ঞানী, ফরেনসিক",
        "বিজ্ঞানী, সামুদ্রিক",
        "বিজ্ঞানী, শারীরবৃত্তীয়",
        "বিজ্ঞানী, পণ্য/প্রক্রিয়া উন্নয়ন",
        "বিজ্ঞানী, গবেষণা (জীবন বিজ্ঞান)",
        "বিজ্ঞানী, গবেষণা (গণিত)",
        "বিজ্ঞানী, গবেষণা (চিকিৎসা)",
        "বিজ্ঞানী, গবেষণা (ভৌত বিজ্ঞান)",
        "বিজ্ঞানী, জলের গুণমান",
        "মাধ্যমিক বিদ্যালয়ের শিক্ষক",
        "সচিব/প্রশাসক",
        "সচিব, কোম্পানি",
        "সিসমিক ইন্টারপ্রেটার",
        "সিনিয়র ট্যাক্স পেশাদার/কর পরিদর্শক",
        "সেট ডিজাইনার",
        "জাহাজ ব্রোকার",
        "প্রকল্প প্রকৌশলী",
        "সামাজিক গবেষণা কর্মকর্তা, সরকার",
        "সমাজ গবেষক",
        "সমাজ কর্মী",
        "সফটওয়্যার ইঞ্জিনিয়ার",
        "মৃত্তিকা বিজ্ঞানী",
        "উকিল",
        "সলিসিটর, স্কটল্যান্ড",
        "সাউন্ড টেকনিশিয়ান, ব্রডকাস্টিং/ফিল্ম/ভিডিও",
        "বিশেষ শিক্ষাগত প্রয়োজন শিক্ষক",
        "বিশেষ প্রভাব শিল্পী",
        "স্পিচ অ্যান্ড ল্যাঙ্গুয়েজ থেরাপিস্ট",
        "ক্রীড়া এবং ব্যায়াম মনোবিজ্ঞানী",
        "ক্রীড়া প্রশাসক",
        "ক্রীড়াশিক্ষক",
        "ক্রীড়া উন্নয়ন কর্মকর্তা",
        "ক্রীড়া থেরাপিস্ট",
        "পর্যায় ম্যানেজার",
        "পরিসংখ্যানবিদ",
        "স্ট্রাকচারাল ইঞ্জিনিয়ার",
        "সাব",
        "সার্জন",
        "জরিপকারী, বিল্ডিং",
        "সার্ভেয়ার, বিল্ডিং কন্ট্রোল",
        "সার্ভেয়ার, বাণিজ্যিক/আবাসিক",
        "জরিপকারী, হাইড্রোগ্রাফিক",
        "জরিপকারী, বীমা",
        "জরিপকারী, ভূমি/জিওম্যাটিক্স",
        "জরিপকারী, খনিজ",
        "জরিপকারী, খনির",
        "জরিপকারী, পরিকল্পনা ও উন্নয়ন",
        "জরিপকারী, পরিমাণ",
        "জরিপকারী, গ্রামীণ অনুশীলন",
        "সিস্টেম বিশ্লেষক",
        "সিস্টেম ডেভেলপার",
        "কর উপদেষ্টা",
        "কর পরিদর্শক",
        "শিক্ষক, বয়স্ক শিক্ষা",
        "শিক্ষক, প্রারম্ভিক বছর / প্রাক",
        "শিক্ষক, ইংরেজি একটি বিদেশী ভাষা হিসাবে",
        "শিক্ষক, সঙ্গীত",
        "শিক্ষক, প্রাথমিক বিদ্যালয়",
        "শিক্ষক, মাধ্যমিক বিদ্যালয়",
        "শিক্ষক, বিশেষ শিক্ষাগত প্রয়োজন",
        "পড়ানো ল্যাবরেটরি টেকনিশিয়ান",
        "প্রযুক্তিগত লেখক",
        "প্রযুক্তিগত ব্রুয়ার",
        "প্রযুক্তিগত বিক্রয় প্রকৌশলী",
        "টিইএফএল শিক্ষক",
        "টেলিযোগাযোগ গবেষক",
        "টেলিভিশন ক্যামেরা অপারেটর",
        "টেলিভিশন ফ্লোর ম্যানেজার",
        "টেলিভিশন প্রযোজনা সহকারী",
        "টেলিভিশন/ফিল্ম/ভিডিও প্রযোজক",
        "টেক্সটাইল ডিজাইনার",
        "থিয়েটার ডিরেক্টর",
        "থিয়েটার ম্যানেজার",
        "থিয়েটার স্টেজ ম্যানেজার",
        "থিম পার্ক ম্যানেজার",
        "থেরাপিউটিক রেডিওগ্রাফার",
        "থেরাপিস্ট, শিল্প",
        "থেরাপিস্ট, নাটক",
        "থেরাপিস্ট, উদ্যানবিদ্যা",
        "থেরাপিস্ট, সঙ্গীত",
        "থেরাপিস্ট, পুষ্টি",
        "থেরাপিস্ট, পেশাগত",
        "থেরাপিস্ট, বক্তৃতা এবং ভাষা",
        "থেরাপিস্ট, খেলাধুলা",
        "ট্যুর ব্যবস্থাপক",
        "পর্যটন কর্মকর্তা",
        "পর্যটন তথ্য কেন্দ্র ব্যবস্থাপক",
        "শহর পরিকল্পনাকারী",
        "বিষাক্ত বিশেষজ্ঞ",
        "ট্রেড মার্ক অ্যাটর্নি",
        "ট্রেড ইউনিয়ন গবেষণা কর্মকর্তা",
        "ট্রেডিং স্ট্যান্ডার্ড অফিসার",
        "প্রশিক্ষণ ও উন্নয়ন কর্মকর্তা",
        "অনুবাদক",
        "পরিবহন পরিকল্পনাকারী",
        "ট্রাভেল এজেন্সি ম্যানেজার",
        "বৃক্ষ সার্জন",
        "পশুচিকিৎসক",
        "ভিডিও এডিটর",
        "ভিজ্যুয়াল মার্চেন্ডাইজার",
        "স্বেচ্ছাসেবক সমন্বয়কারী",
        "ওয়ার্ডেন/রেঞ্জার",
        "গুদাম ম্যানেজার",
        "বর্জ্য ব্যবস্থাপনা কর্মকর্তা",
        "জল প্রকৌশলী",
        "পানির গুণমান বিজ্ঞানী",
        "ওয়েব ডিজাইনার",
        "ওয়েলসাইট ভূতত্ত্ববিদ",
        "লেখক",
        "যুবকর্মী",
    )



from typing import Dict

from .. import Provider as LoremProvider


class Provider(LoremProvider):
    """Lorem provider for mk_MK locale (Macedonian)."""

    word_list = (
        "луѓе",
        "живот",
        "работа",
        "време",
        "место",
        "свет",
        "земја",
        "град",
        "куќа",
        "пат",
        "ден",
        "ноќ",
        "вода",
        "сонце",
        "небо",
        "страна",
        "рака",
        "очи",
        "глава",
        "срце",
        "деца",
        "мајка",
        "татко",
        "брат",
        "сестра",
        "пријател",
        "љубов",
        "среќа",
        "мир",
        "народ",
        "слобода",
        "правда",
        "вистина",
        "убавина",
        "природа",
        "шума",
        "река",
        "море",
        "планина",
        "село",
        "улица",
        "прозорец",
        "врата",
        "книга",
        "музика",
        "песна",
        "уметност",
        "историја",
        "култура",
        "јазик",
        "зборови",
        "мисла",
        "чувство",
        "сон",
        "надеж",
        "иднина",
        "минато",
        "сегашност",
        "знаење",
        "образование",
        "наука",
        "технологија",
        "компјутер",
        "интернет",
        "телефон",
        "информација",
        "вест",
        "проблем",
        "решение",
        "прашање",
        "одговор",
        "можност",
        "промена",
        "развиток",
        "успех",
        "цел",
        "план",
        "идеја",
        "создавање",
        "градење",
        "заедница",
        "семејство",
        "пријателство",
        "соработка",
        "разбирање",
        "доверба",
        "поддршка",
        "помош",
        "грижа",
        "здравје",
        "храна",
        "леб",
        "вино",
        "сол",
        "светлина",
        "темнина",
        "топлина",
        "студ",
        "дожд",
        "ветер",
        "снег",
        "оган",
        "земјиште",
        "камен",
        "песок",
        "злато",
        "сребро",
        "железо",
        "дрво",
        "лист",
        "цвет",
        "трева",
        "жито",
        "овошје",
        "зеленчук",
        "риба",
        "птица",
        "коњ",
        "куче",
        "мачка",
        "волк",
        "мечка",
        "орел",
        "небесен",
        "длабок",
        "висок",
        "широк",
        "тесен",
        "голем",
        "мал",
        "стар",
        "млад",
        "убав",
        "добар",
        "лош",
        "силен",
        "слаб",
        "брз",
        "бавен",
        "тивок",
        "гласен",
        "светол",
        "темен",
        "топол",
        "ладен",
        "тежок",
        "лесен",
        "трчи",
        "оди",
        "лета",
        "пее",
        "зборува",
        "пишува",
        "чита",
        "гледа",
        "слуша",
        "мисли",
        "сака",
        "знае",
        "може",
        "дава",
        "зема",
        "прави",
        "живее",
        "расте",
        "учи",
        "работи",
        "игра",
        "спие",
        "јаде",
        "пие",
        "смее",
        "плаче",
        "денес",
        "утре",
        "вчера",
        "сега",
        "тука",
        "таму",
        "многу",
        "малку",
        "секогаш",
        "никогаш",
        "брзо",
        "полека",
        "добро",
        "лошо",
        "заедно",
        "сам",
        "прво",
        "потоа",
        "пред",
        "после",
        "горе",
        "доле",
        "лево",
        "десно",
        "внатре",
        "надвор",
        "зошто",
        "затоа",
        "ако",
        "кога",
        "каде",
        "кој",
        "што",
    )

    parts_of_speech: Dict[str, tuple] = {}



from .. import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``uk_UA`` locale.
    Source for rules for bban format:
    https://bank.gov.ua/en/iban
    Banks list:
    https://ubanks.com.ua/adr/
    """

    bban_format = "#" * 27
    country_code = "UA"
    banks = (
        "izibank",
        "monobank",
        "O.Bank",
        "sportbank",
        "А-Банк",
        "Агропросперіс Банк",
        "АкордБанк",
        "Альтбанк",
        "Асвіо Банк",
        "Банк 3/4",
        "Банк Авангард",
        "Банк Альянс",
        "Банк Власний Рахунок",
        "Банк Восток",
        "Банк інвестицій та заощаджень",
        "Банк Кредит Дніпро",
        "Банк Портал",
        "Банк Український Капітал",
        "Банк Фамільний",
        "БТА Банк",
        "Глобус",
        "Грант",
        "Дойче Банк ДБУ",
        "Європейський Промисловий Банк",
        "Ідея Банк",
        "ІНГ Банк Україна",
        "Індустріалбанк",
        "Кліринговий Дім",
        "Комінбанк",
        "КомІнвестБанк",
        "Кредит Європа Банк",
        "Кредитвест Банк",
        "Креді Агріколь",
        "Кредобанк",
        "Кристалбанк",
        "Львів",
        "МетаБанк",
        "Міжнародний Інвестиційний Банк",
        "Мотор-Банк",
        "МТБ Банк",
        "Національний банк України",
        "Оксі Банк",
        "ОТП Банк",
        "Ощадбанк",
        "Перший Інвестиційний Банк",
        "Перший Український Міжнародний Банк",
        "Південний",
        "Піреус Банк",
        "Полікомбанк",
        "Полтава-Банк",
        "Правекс Банк",
        "ПриватБанк",
        "ПроКредит Банк",
        "Радабанк",
        "Райффайзен Банк",
        "РВС Банк",
        "СЕБ Корпоративний Банк",
        "Сенс Банк",
        "Сітібанк",
        "Скай Банк",
        "ТАСкомбанк",
        "Траст-капітал",
        "Український банк реконструкції та розвитку",
        "Укргазбанк",
        "Укрексімбанк",
        "УкрСиббанк",
        "Універсал Банк",
        "Юнекс Банк",
    )



from collections import OrderedDict
from typing import Dict, List, Optional

from ... import ElementsType
from ..en import Provider as AddressProvider

# https://en.wikipedia.org/wiki/Addresses_in_Malaysia


class Provider(AddressProvider):
    # 'Bandar' and 'Taman' are the most common township prefix
    # https://en.wikipedia.org/wiki/Template:Greater_Kuala_Lumpur > Townships
    # https://en.wikipedia.org/wiki/Template:Johor > Townships
    # https://en.wikipedia.org/wiki/Template:Kedah > Townships
    # https://en.wikipedia.org/wiki/Template:Kelantan > Townships
    # https://en.wikipedia.org/wiki/Template:Melaka > Townships
    # https://en.wikipedia.org/wiki/Template:Negeri_Sembilan > Townships
    # https://en.wikipedia.org/wiki/Template:Perak > Townships
    # https://en.wikipedia.org/wiki/Template:Penang > Townships
    # https://en.wikipedia.org/wiki/Template:Selangor > Townships
    # https://en.wikipedia.org/wiki/Template:Terengganu > Townships

    city_prefixes = (
        "Alam",
        "Apartment",
        "Ara",
        "Bandar",
        "Bandar",
        "Bandar",
        "Bandar",
        "Bandar",
        "Bandar",
        "Bandar Bukit",
        "Bandar Seri",
        "Bandar Sri",
        "Bandar Baru",
        "Batu",
        "Bukit",
        "Desa",
        "Damansara",
        "Kampung",
        "Kampung Baru",
        "Kampung Baru",
        "Kondominium",
        "Kota",
        "Laman",
        "Lembah",
        "Medan",
        "Pandan",
        "Pangsapuri",
        "Petaling",
        "Puncak",
        "Seri",
        "Sri",
        "Taman",
        "Taman",
        "Taman",
        "Taman",
        "Taman",
        "Taman",
        "Taman Desa",
    )

    city_suffixes = (
        "Aman",
        "Amanjaya",
        "Anggerik",
        "Angkasa",
        "Antarabangsa",
        "Awan",
        "Bahagia",
        "Bangsar",
        "Baru",
        "Belakong",
        "Bendahara",
        "Bestari",
        "Bintang",
        "Brickfields",
        "Casa",
        "Changkat",
        "Country Heights",
        "Damansara",
        "Damai",
        "Dato Harun",
        "Delima",
        "Duta",
        "Flora",
        "Gembira",
        "Genting",
        "Harmoni",
        "Hartamas",
        "Impian",
        "Indah",
        "Intan",
        "Jasa",
        "Jaya",
        "Keramat",
        "Kerinchi",
        "Kiara",
        "Kinrara",
        "Kuchai",
        "Laksamana",
        "Mahkota",
        "Maluri",
        "Manggis",
        "Maxwell",
        "Medan",
        "Melawati",
        "Menjalara",
        "Meru",
        "Mulia",
        "Mutiara",
        "Pahlawan",
        "Perdana",
        "Pertama",
        "Permai",
        "Pelangi",
        "Petaling",
        "Pinang",
        "Puchong",
        "Puteri",
        "Putra",
        "Rahman",
        "Rahmat",
        "Raya",
        "Razak",
        "Ria",
        "Saujana",
        "Segambut",
        "Selamat",
        "Selatan",
        "Semarak",
        "Sentosa",
        "Seputeh",
        "Setapak",
        "Setia Jaya",
        "Sinar",
        "Sungai Besi",
        "Sungai Buaya",
        "Sungai Long",
        "Suria",
        "Tasik Puteri",
        "Tengah",
        "Timur",
        "Tinggi",
        "Tropika",
        "Tun Hussein Onn",
        "Tun Perak",
        "Tunku",
        "Ulu",
        "Utama",
        "Utara",
        "Wangi",
    )

    # https://en.wikipedia.org/wiki/States_and_federal_territories_of_Malaysia
    states: Dict[str, List[str]] = {
        "JHR": ["Johor Darul Ta'zim", "Johor"],
        "KDH": ["Kedah Darul Aman", "Kedah"],
        "KTN": ["Kelantan Darul Naim", "Kelantan"],
        "KUL": ["KL", "Kuala Lumpur", "WP Kuala Lumpur"],
        "LBN": ["Labuan"],
        "MLK": ["Malacca", "Melaka"],
        "NSN": ["Negeri Sembilan Darul Khusus", "Negeri Sembilan"],
        "PHG": ["Pahang Darul Makmur", "Pahang"],
        "PNG": ["Penang", "Pulau Pinang"],
        "PRK": ["Perak Darul Ridzuan", "Perak"],
        "PLS": ["Perlis Indera Kayangan", "Perlis"],
        "PJY": ["Putrajaya"],
        "SBH": ["Sabah"],
        "SWK": ["Sarawak"],
        "SGR": ["Selangor Darul Ehsan", "Selangor"],
        "TRG": ["Terengganu Darul Iman", "Terengganu"],
    }

    states_postcode = {
        "PLS": [(1000, 2800)],
        "KDH": [(5000, 9810)],
        "PNG": [(10000, 14400)],
        "KTN": [(15000, 18500)],
        "TRG": [(20000, 24300)],
        "PHG": [
            (25000, 28800),
            (39000, 39200),
            (49000, 69000),
        ],
        "PRK": [(30000, 36810)],
        "SGR": [(40000, 48300), (63000, 68100)],
        "KUL": [(50000, 60000)],
        "PJY": [(62000, 62988)],
        "NSN": [(70000, 73509)],
        "MLK": [(75000, 78309)],
        "JHR": [(79000, 86900)],
        "LBN": [(87000, 87033)],
        "SBH": [(88000, 91309)],
        "SWK": [(93000, 98859)],
    }

    city_prefix_abbrs: ElementsType[str] = (
        "SS",
        "Seksyen ",
        "PJS",
        "PJU",
        "USJ ",
    )

    def city_prefix_abbr(self) -> str:
        return self.random_element(self.city_prefix_abbrs)

    city_formats: ElementsType[str] = (
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix}} {{city_suffix}}",
        "{{city_prefix_abbr}}%",
        "{{city_prefix_abbr}}%#",
        "{{city_prefix_abbr}}%#?",
    )

    def city(self) -> str:
        pattern: str = self.bothify(self.random_element(self.city_formats))
        return self.generator.parse(pattern)

    # https://en.wikipedia.org/wiki/List_of_roads_in_Kuala_Lumpur#Standard_translations
    street_prefixes: ElementsType[str] = [
        "Jln",
        "Jln",
        "Jalan",
        "Jalan",
        "Jalan",
        "Lorong",
    ]

    def street_prefix(self) -> str:
        return self.random_element(self.street_prefixes)

    # https://en.wikipedia.org/wiki/List_of_roads_in_Kuala_Lumpur
    # https://en.wikipedia.org/wiki/List_of_roads_in_Ipoh
    # https://en.wikipedia.org/wiki/Transportation_in_Seremban#Inner_city_roads
    # https://en.wikipedia.org/wiki/List_of_streets_in_George_Town,_Penang
    street_suffixes: ElementsType[str] = [
        "Air Itam",
        "Alor",
        "Ampang",
        "Ampang Hilir",
        "Anson",
        "Ariffin",
        "Bangsar",
        "Baru",
        "Bellamy",
        "Birch",
        "Bijih Timah",
        "Bukit Aman",
        "Bukit Bintang",
        "Bukit Petaling",
        "Bukit Tunku",
        "Cantonment",
        "Cenderawasih",
        "Chan Sow Lin",
        "Chow Kit",
        "Cinta",
        "Cochrane",
        "Conlay",
        "D. S. Ramanathan",
        "Damansara",
        "Dang Wangi",
        "Davis",
        "Dewan Bahasa",
        "Dato Abdul Rahman",
        "Dato'Keramat",
        "Dato' Maharaja Lela",
        "Doraisamy",
        "Eaton",
        "Faraday",
        "Galloway",
        "Genting Klang",
        "Gereja",
        "Hang Jebat",
        "Hang Kasturi",
        "Hang Lekir",
        "Hang Lekiu",
        "Hang Tuah",
        "Hospital",
        "Imbi",
        "Istana",
        "Jelutong",
        "Kampung Attap",
        "Kebun Bunga",
        "Kedah",
        "Keliling",
        "Kia Peng",
        "Kinabalu",
        "Kuala Kangsar",
        "Kuching",
        "Ledang",
        "Lembah Permai",
        "Loke Yew",
        "Lt. Adnan",
        "Lumba Kuda",
        "Madras",
        "Magazine",
        "Maharajalela",
        "Masjid",
        "Maxwell",
        "Mohana Chandran",
        "Muda",
        "P. Ramlee",
        "Padang Kota Lama",
        "Pahang",
        "Pantai Baharu",
        "Parlimen",
        "Pasar",
        "Pasar Besar",
        "Perak",
        "Perdana",
        "Petaling",
        "Prangin",
        "Pudu",
        "Pudu Lama",
        "Raja",
        "Raja Abdullah",
        "Raja Chulan",
        "Raja Laut",
        "Rakyat",
        "Residensi",
        "Robson",
        "S.P. Seenivasagam",
        "Samarahan 1",
        "Selamat",
        "Sempadan",
        "Sentul",
        "Serian 1",
        "Sasaran",
        "Sin Chee",
        "Sultan Abdul Samad",
        "Sultan Azlan Shah",
        "Sultan Iskandar",
        "Sultan Ismail",
        "Sultan Sulaiman",
        "Sungai Besi",
        "Syed Putra",
        "Tan Cheng Lock",
        "Thambipillay",
        "Tugu",
        "Tuanku Abdul Halim",
        "Tuanku Abdul Rahman",
        "Tun Abdul Razak",
        "Tun Dr Ismail",
        "Tun H S Lee",
        "Tun Ismail",
        "Tun Perak",
        "Tun Razak",
        "Tun Sambanthan",
        "U-Thant",
        "Utama",
        "Vermont",
        "Vivekananda",
        "Wan Kadir",
        "Wesley",
        "Wisma Putra",
        "Yaacob Latif",
        "Yap Ah Loy",
        "Yap Ah Shak",
        "Yap Kwan Seng",
        "Yew",
        "Zaaba",
        "Zainal Abidin",
    ]

    street_name_formats: ElementsType[str] = (
        "{{street_prefix}} %",
        "{{street_prefix}} %/%",
        "{{street_prefix}} %/%#",
        "{{street_prefix}} %/%?",
        "{{street_prefix}} %/%#?",
        "{{street_prefix}} %?",
        "{{street_prefix}} %#?",
        "{{street_prefix}} {{street_suffix}}",
        "{{street_prefix}} {{street_suffix}} %",
        "{{street_prefix}} {{street_suffix}} %/%",
        "{{street_prefix}} {{street_suffix}} %/%#",
        "{{street_prefix}} {{street_suffix}} %/%?",
        "{{street_prefix}} {{street_suffix}} %/%#?",
        "{{street_prefix}} {{street_suffix}} %?",
        "{{street_prefix}} {{street_suffix}} %#?",
    )

    def street_name(self) -> str:
        """
        :example: 'Crist Parks'
        """
        pattern: str = self.bothify(self.random_element(self.street_name_formats))
        return self.generator.parse(pattern)

    building_prefixes: ElementsType[str] = [
        "",
        "",
        "",
        "",
        "",
        "",
        "No. ",
        "No. ",
        "No. ",
        "Lot ",
    ]

    def building_prefix(self) -> str:
        return self.random_element(self.building_prefixes)

    building_number_formats: ElementsType[str] = (
        "%",
        "%",
        "%",
        "%#",
        "%#",
        "%#",
        "%#",
        "%##",
        "%-%",
        "?-##-##",
        "%?-##",
    )

    def building_number(self) -> str:
        return self.bothify(self.random_element(self.building_number_formats))

    street_address_formats: ElementsType[str] = ("{{building_prefix}}{{building_number}}, {{street_name}}",)

    def city_state(self) -> str:
        """Return the complete city address with matching postcode and state

        Example: 55100 Bukit Bintang, Kuala Lumpur
        """
        state: str = self.random_element(self.states.keys())
        postcode = self.postcode_in_state(state)
        city = self.city()
        state_name: str = self.random_element(self.states[state])

        return f"{postcode} {city}, {state_name}"

    # https://en.wikipedia.org/wiki/Addresses_in_Malaysia
    # street number, street name, region, and town/city, state.
    address_formats = OrderedDict((("{{street_address}}, {{city}}, {{city_state}}", 100.0),))

    def city_prefix(self) -> str:
        return self.random_element(self.city_prefixes)

    def administrative_unit(self) -> str:
        return self.random_element(self.states[self.random_element(self.states.keys())])

    state = administrative_unit

    def postcode_in_state(self, state_abbr: Optional[str] = None) -> str:
        """
        :returns: A random postcode within the provided state

        :param state: A state

        Example: 55100
        https://en.wikipedia.org/wiki/Postal_codes_in_Malaysia#States
        """

        if state_abbr is None:
            state_abbr = self.random_element(self.states.keys())

        try:
            # some states have multiple ranges so first pick one, then generate a random postcode
            range = self.generator.random.choice(self.states_postcode[state_abbr])
            postcode = "%d" % (self.generator.random.randint(*range))

            # zero left pad up until desired length (some have length 3 or 4)
            target_postcode_len = 5
            current_postcode_len = len(postcode)
            if current_postcode_len < target_postcode_len:
                pad = target_postcode_len - current_postcode_len
                postcode = f"{'0'*pad}{postcode}"

            return postcode
        except KeyError as e:
            raise KeyError("State Abbreviation not found in list") from e

    def postcode(self) -> str:
        return self.postcode_in_state(None)



from .. import Provider as AutomotiveProvider


class Provider(AutomotiveProvider):
    """Implement automotive provider for ``ar_DZ`` locale.

    Sources:

    - https://en.wikipedia.org/wiki/Vehicle_registration_plates_of_Algeria
    """

    WILAYA_CODES = tuple(f"{i:02d}" for i in range(1, 59))
    VEHICLE_CLASSES = tuple("123456789")

    def license_plate(self) -> str:
        serial = self.numerify("#####")
        vehicle_class = self.random_element(self.VEHICLE_CLASSES)
        year = self.numerify("##")
        wilaya = self.random_element(self.WILAYA_CODES)
        return f"{serial} {vehicle_class}{year} {wilaya}"



import string

from faker.utils import checksums

from .. import Provider as BaseProvider

GSTIN_CHECKSUM_ALPHABET = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"
GSTIN_CHECKSUM_BASE = len(GSTIN_CHECKSUM_ALPHABET)


def calculate_gstin_checksum(gstin_without_checksum: str) -> str:
    """Calculate checksum character for a 14-character GSTIN prefix."""

    factor = 2
    total = 0

    for char in reversed(gstin_without_checksum):
        code_point = GSTIN_CHECKSUM_ALPHABET.index(char)
        addend = factor * code_point
        factor = 1 if factor == 2 else 2
        addend = (addend // GSTIN_CHECKSUM_BASE) + (addend % GSTIN_CHECKSUM_BASE)
        total += addend

    remainder = total % GSTIN_CHECKSUM_BASE
    check_code_point = (GSTIN_CHECKSUM_BASE - remainder) % GSTIN_CHECKSUM_BASE
    return GSTIN_CHECKSUM_ALPHABET[check_code_point]


class Provider(BaseProvider):
    """
    Faker provider for Indian Identifiers
    """

    aadhaar_id_formats = ("%##########",)
    pan_type_chars = tuple("CPHFATBLJG")
    gstin_entity_code_chars = tuple("123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ")

    def aadhaar_id(self) -> str:
        """
        Aadhaar is a 12 digit person identifier generated for residents of
        India.
        Details: https://en.wikipedia.org/wiki/Aadhaar
        Official Website: https://uidai.gov.in/my-aadhaar/about-your-aadhaar.html
        """

        aadhaar_digits = self.numerify(self.random_element(self.aadhaar_id_formats))
        checksum = checksums.calculate_luhn(int(aadhaar_digits))

        aadhaar_number = f"{aadhaar_digits}{checksum}"

        return aadhaar_number

    def pan(self) -> str:
        """
        Permanent Account Number (PAN) is a 10 character alphanumeric tax
        identifier issued in India.
        Details: https://en.wikipedia.org/wiki/Permanent_account_number
        """

        first_three = self.lexify("???", letters=string.ascii_uppercase)
        pan_type = self.random_element(self.pan_type_chars)
        holder_initial = self.random_uppercase_letter()
        serial = self.numerify("####")
        check_char = self.random_uppercase_letter()

        return f"{first_three}{pan_type}{holder_initial}{serial}{check_char}"

    def gstin(self) -> str:
        """
        Goods and Services Tax Identification Number (GSTIN) is a 15 character
        identifier used in India for GST registration.
        Details: https://en.wikipedia.org/wiki/Goods_and_Services_Tax_(India)
        """

        state_code = f"{self.random_int(min=1, max=38):02d}"
        entity_code = self.random_element(self.gstin_entity_code_chars)
        gstin_without_checksum = f"{state_code}{self.pan()}{entity_code}Z"
        checksum = calculate_gstin_checksum(gstin_without_checksum)

        return f"{gstin_without_checksum}{checksum}"



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    # Source references (accessed 2026-04-18):
    # - https://mr.wikipedia.org/wiki/वार: (Marathi weekday naming)
    # - https://mr.wikipedia.org/wiki/महिने: (Marathi month naming)
    # Data is normalized for synthetic locale generation.
    DAY_NAMES = {
        "0": "रविवार",
        "1": "सोमवार",
        "2": "मंगळवार",
        "3": "बुधवार",
        "4": "गुरुवार",
        "5": "शुक्रवार",
        "6": "शनिवार",
    }

    MONTH_NAMES = {
        "01": "जानेवारी",
        "02": "फेब्रुवारी",
        "03": "मार्च",
        "04": "एप्रिल",
        "05": "मे",
        "06": "जून",
        "07": "जुलै",
        "08": "ऑगस्ट",
        "09": "सप्टेंबर",
        "10": "ऑक्टोबर",
        "11": "नोव्हेंबर",
        "12": "डिसेंबर",
    }

    def day_of_week(self) -> str:
        day = self.date("%w")
        return self.DAY_NAMES[day]

    def month_name(self) -> str:
        month = self.month()
        return self.MONTH_NAMES[month]



from .. import Provider as CompanyProvider


class Provider(CompanyProvider):
    """Company provider for fr_DZ locale (Algeria, French-language)."""

    # Sources:
    #   - https://www.commerce.gov.dz/fr/choix-de-la-forme-juridique-de-votre-entreprise-1
    formats = (
        "{{last_name}} {{company_suffix}}",
        "{{last_name}} {{last_name}} {{company_suffix}}",
        "{{last_name}} et Associés",
        "{{last_name}} & {{last_name}}",
        "{{last_name}}",
    )

    company_suffixes = (
        "SARL",
        "S.A.R.L.",
        "SPA",
        "S.P.A.",
        "EURL",
        "E.U.R.L.",
        "SNC",
        "S.N.C.",
        "SCS",
        "S.C.S.",
        "SEM",
        "EP",
        "EPIC",
    )

    catch_phrase_formats = ("{{catch_phrase_noun}} {{catch_phrase_verb}} {{catch_phrase_attribute}}",)

    nouns = (
        "la qualité",
        "l'excellence",
        "la confiance",
        "le développement",
        "la croissance",
        "l'innovation",
        "la performance",
        "le progrès",
        "la réussite",
        "le service",
        "la sécurité",
        "le partenariat",
    )

    verbs = (
        "de réussir",
        "d'avancer",
        "d'évoluer",
        "de se développer",
        "d'innover",
        "de progresser",
        "d'investir",
        "de construire",
        "d'atteindre vos objectifs",
        "de concrétiser vos projets",
    )

    attributes = (
        "ensemble",
        "durablement",
        "efficacement",
        "avec confiance",
        "en toute sécurité",
        "pour demain",
        "pour l'avenir",
        "au service du pays",
        "au cœur du développement",
        "avec excellence",
    )

    def catch_phrase(self) -> str:
        pattern: str = self.random_element(self.catch_phrase_formats)
        return self.generator.parse(pattern)

    def catch_phrase_noun(self) -> str:
        return self.random_element(self.nouns)

    def catch_phrase_verb(self) -> str:
        return self.random_element(self.verbs)

    def catch_phrase_attribute(self) -> str:
        return self.random_element(self.attributes)



# Data source
#
# Ethiopia national numbering plan: mobile prefixes 09 (Ethio Telecom) and 07 (Safaricom Ethiopia);
# international format country code +251 with trunk prefix 0 omitted.
# https://en.wikipedia.org/wiki/Telephone_numbers_in_Ethiopia (retrieved 2026-03-29)

from .. import Provider as PhoneNumberProvider


class Provider(PhoneNumberProvider):
    formats = (
        "09########",
        "+2519########",
        "07########",
        "+2517########",
    )



from typing import List

from .. import Provider as BaseProvider


class Provider(BaseProvider):
    INVALID_SSN_TYPE = "INVALID_SSN"
    SSN_TYPE = "SSN"
    ITIN_TYPE = "ITIN"
    EIN_TYPE = "EIN"

    def itin(self) -> str:
        """Generate a random United States Individual Taxpayer Identification Number (ITIN).

        An United States Individual Taxpayer Identification Number
        (ITIN) is a tax processing number issued by the Internal
        Revenue Service. It is a nine-digit number that always begins
        with the number 9 and has a range of 70-88 in the fourth and
        fifth digit. Effective April 12, 2011, the range was extended
        to include 900-70-0000 through 999-88-9999, 900-90-0000
        through 999-92-9999 and 900-94-0000 through 999-99-9999.
        https://www.irs.gov/individuals/international-taxpayers/general-itin-information
        """

        area = self.random_int(min=900, max=999)
        serial = self.random_int(min=0, max=9999)

        # The group number must be between 70 and 99 inclusively but not 89 or 93
        group: int = self.random_element([x for x in range(70, 100) if x not in [89, 93]])

        itin = f"{area:03d}-{group:02d}-{serial:04d}"
        return itin

    def ein(self) -> str:
        """Generate a random United States Employer Identification Number (EIN).

        An United States An Employer Identification Number (EIN) is
        also known as a Federal Tax Identification Number, and is
        used to identify a business entity. EINs follow a format of a
        two-digit prefix followed by a hyphen and a seven-digit sequence:
        ##-######

        https://www.irs.gov/businesses/small-businesses-self-employed/employer-id-numbers
        """

        # Only certain EIN Prefix values are assigned:
        #
        # https://www.irs.gov/businesses/small-businesses-self-employed/how-eins-are-assigned-and-valid-ein-prefixes

        ein_prefix_choices: List[str] = [
            "01",
            "02",
            "03",
            "04",
            "05",
            "06",
            "10",
            "11",
            "12",
            "13",
            "14",
            "15",
            "16",
            "20",
            "21",
            "22",
            "23",
            "24",
            "25",
            "26",
            "27",
            "30",
            "31",
            "32",
            "33",
            "34",
            "35",
            "36",
            "37",
            "38",
            "39",
            "40",
            "41",
            "42",
            "43",
            "44",
            "45",
            "46",
            "47",
            "48",
            "50",
            "51",
            "52",
            "53",
            "54",
            "55",
            "56",
            "57",
            "58",
            "59",
            "60",
            "61",
            "62",
            "63",
            "64",
            "65",
            "66",
            "67",
            "68",
            "71",
            "72",
            "73",
            "74",
            "75",
            "76",
            "77",
            "80",
            "81",
            "82",
            "83",
            "84",
            "85",
            "86",
            "87",
            "88",
            "90",
            "91",
            "92",
            "93",
            "94",
            "95",
            "98",
            "99",
        ]

        ein_prefix: str = self.random_element(ein_prefix_choices)
        sequence = self.random_int(min=0, max=9999999)

        ein = f"{ein_prefix:s}-{sequence:07d}"
        return ein

    def invalid_ssn(self) -> str:
        """Generate a random invalid United States Social Security Identification Number (SSN).

        Invalid SSNs have the following characteristics:
        Cannot begin with the number 9
        Cannot begin with 666 in positions 1 - 3
        Cannot begin with 000 in positions 1 - 3
        Cannot contain 00 in positions 4 - 5
        Cannot contain 0000 in positions 6 - 9

        https://www.ssa.gov/kc/SSAFactSheet--IssuingSSNs.pdf

        Additionally, return an invalid SSN that is NOT a valid ITIN by excluding certain ITIN related "group" values
        """
        itin_group_numbers = [
            70,
            71,
            72,
            73,
            74,
            75,
            76,
            77,
            78,
            79,
            80,
            81,
            82,
            83,
            84,
            85,
            86,
            87,
            88,
            90,
            91,
            92,
            94,
            95,
            96,
            97,
            98,
            99,
        ]
        area = self.random_int(min=0, max=999)
        if area < 900 and area not in {666, 0}:
            random_group_or_serial = self.random_int(min=1, max=1000)
            if random_group_or_serial <= 500:
                group = 0
                serial = self.random_int(0, 9999)
            else:
                group = self.random_int(0, 99)
                serial = 0
        elif area in {666, 0}:
            group = self.random_int(0, 99)
            serial = self.random_int(0, 9999)
        else:
            group = self.random_element([x for x in range(0, 100) if x not in itin_group_numbers])
            serial = self.random_int(0, 9999)

        invalid_ssn = f"{area:03d}-{group:02d}-{serial:04d}"
        return invalid_ssn

    def ssn(self, taxpayer_identification_number_type: str = SSN_TYPE) -> str:
        """Generate a random United States Taxpayer Identification Number of the specified type.

        If no type is specified, a US SSN is returned.
        """

        if taxpayer_identification_number_type == self.ITIN_TYPE:
            return self.itin()
        elif taxpayer_identification_number_type == self.EIN_TYPE:
            return self.ein()
        elif taxpayer_identification_number_type == self.INVALID_SSN_TYPE:
            return self.invalid_ssn()
        elif taxpayer_identification_number_type == self.SSN_TYPE:
            # Certain numbers are invalid for United States Social Security
            # Numbers. The area (first 3 digits) cannot be 666 or 900-999.
            # The group number (middle digits) cannot be 00. The serial
            # (last 4 digits) cannot be 0000.

            area = self.random_int(min=1, max=899)
            if area == 666:
                area += 1
            group = self.random_int(1, 99)
            serial = self.random_int(1, 9999)

            ssn = f"{area:03d}-{group:02d}-{serial:04d}"
            return ssn

        else:
            raise ValueError(
                "taxpayer_identification_number_type must be one of 'SSN', 'EIN', 'ITIN', or 'INVALID_SSN'."
            )



from collections import OrderedDict

from faker.utils.decorators import lowercase, slugify

from .. import Provider as InternetProvider


class Provider(InternetProvider):
    """
    Provider for internet stuff for en_PH locale

    Free email domains are based on anecdotal evidence and experience. Available TLDs are based on the listed sources.
    Because of the local company naming scheme, a custom generator is needed to output convincing company domains.

    Sources:
    - https://en.wikipedia.org/wiki/.ph
    """

    tlds = (
        "com",
        "net",
        "org",
        "ph",
        "com.ph",
        "net.ph",
        "org.ph",
    )
    safe_email_tlds = tlds
    free_email_domains = (
        "gmail.com",
        "yahoo.com",
        "zohomail.com",
    )
    email_formats = OrderedDict(
        [
            ("{{user_name}}@{{domain_name}}", 0.75),
            ("{{user_name}}@{{free_email_domain}}", 0.25),
        ]
    )

    @lowercase
    @slugify
    def domain_word(self) -> str:
        check = self.random_int(0, 99)
        if check % 100 < 40:
            company_acronym = self.generator.format("random_company_acronym")
            if len(company_acronym) == 2:
                company_type = self.generator.format("company_type")
                return company_acronym + company_type
            else:
                return company_acronym
        else:
            if check % 2 == 0:
                name_part = self.generator.format("last_name")
            else:
                name_part = self.generator.format("random_company_adjective")
            company_noun_chain = self.generator.format("random_company_noun_chain")
            company_nouns = company_noun_chain.split(" ")
            if len(company_nouns) == 1:
                return name_part + company_noun_chain
            else:
                company_type = self.generator.format("company_type")
                company_elements = [name_part] + company_nouns
                acronym = "".join([word[0] for word in company_elements])
                return acronym + company_type



from collections import OrderedDict

from ..en import Provider as AddressProvider


class Provider(AddressProvider):
    city_prefixes = ("North", "East", "West", "South", "New", "Lake", "Port")
    city_suffixes = (
        "town",
        "ton",
        "land",
        "ville",
        "berg",
        "burgh",
        "borough",
        "bury",
        "view",
        "port",
        "mouth",
        "stad",
        "furt",
        "chester",
        "mouth",
        "fort",
        "haven",
        "side",
        "shire",
    )
    counties = (
        "Bedfordshire",
        "Buckinghamshire",
        "Cambridgeshire",
        "Cheshire",
        "Cleveland",
        "Cornwall",
        "Cumbria",
        "Derbyshire",
        "Devon",
        "Dorset",
        "Durham",
        "East Sussex",
        "Essex",
        "Gloucestershire",
        "Greater London",
        "Greater Manchester",
        "Hampshire",
        "Hertfordshire",
        "Kent",
        "Lancashire",
        "Leicestershire",
        "Lincolnshire",
        "Merseyside",
        "Norfolk",
        "North Yorkshire",
        "Northamptonshire",
        "Northumberland",
        "Nottinghamshire",
        "Oxfordshire",
        "Shropshire",
        "Somerset",
        "South Yorkshire",
        "Staffordshire",
        "Suffolk",
        "Surrey",
        "Tyne and Wear",
        "Warwickshire",
        "West Berkshire",
        "West Midlands",
        "West Sussex",
        "West Yorkshire",
        "Wiltshire",
        "Worcestershire",
        "Flintshire",
        "Glamorgan",
        "Merionethshire",
        "Monmouthshire",
        "Montgomeryshire",
        "Pembrokeshire",
        "Radnorshire",
        "Anglesey",
        "Breconshire",
        "Caernarvonshire",
        "Cardiganshire",
        "Carmarthenshire",
        "Denbighshire",
        "Aberdeen City",
        "Aberdeenshire",
        "Angus",
        "Argyll and Bute",
        "City of Edinburgh",
        "Clackmannanshire",
        "Dumfries and Galloway",
        "Dundee City",
        "East Ayrshire",
        "East Dunbartonshire",
        "East Lothian",
        "East Renfrewshire",
        "Eilean Siar",
        "Falkirk",
        "Fife",
        "Glasgow City",
        "Highland",
        "Inverclyde",
        "Midlothian",
        "Moray",
        "North Ayrshire",
        "North Lanarkshire",
        "Orkney Islands",
        "Perth and Kinross",
        "Renfrewshire",
        "Scottish Borders",
        "Shetland Islands",
        "South Ayrshire",
        "South Lanarkshire",
        "Stirling",
        "West Dunbartonshire",
        "West Lothian",
        "Antrim",
        "Armagh",
        "Down",
        "Fermanagh",
        "Derry and Londonderry",
        "Tyrone",
    )
    building_number_formats = ("#", "##", "###")
    street_suffixes = (
        "Alley",
        "Avenue",
        "Branch",
        "Bridge",
        "Brook",
        "Brooks",
        "Burg",
        "Burgs",
        "Bypass",
        "Camp",
        "Canyon",
        "Cape",
        "Causeway",
        "Center",
        "Centers",
        "Circle",
        "Circles",
        "Cliff",
        "Cliffs",
        "Club",
        "Common",
        "Corner",
        "Corners",
        "Course",
        "Court",
        "Courts",
        "Cove",
        "Coves",
        "Creek",
        "Crescent",
        "Crest",
        "Crossing",
        "Crossroad",
        "Curve",
        "Dale",
        "Dam",
        "Divide",
        "Drive",
        "Drive",
        "Drives",
        "Estate",
        "Estates",
        "Expressway",
        "Extension",
        "Extensions",
        "Fall",
        "Falls",
        "Ferry",
        "Field",
        "Fields",
        "Flat",
        "Flats",
        "Ford",
        "Fords",
        "Forest",
        "Forge",
        "Forges",
        "Fork",
        "Forks",
        "Fort",
        "Freeway",
        "Garden",
        "Gardens",
        "Gateway",
        "Glen",
        "Glens",
        "Green",
        "Greens",
        "Grove",
        "Groves",
        "Harbor",
        "Harbors",
        "Haven",
        "Heights",
        "Highway",
        "Hill",
        "Hills",
        "Hollow",
        "Inlet",
        "Inlet",
        "Island",
        "Island",
        "Islands",
        "Islands",
        "Isle",
        "Isle",
        "Junction",
        "Junctions",
        "Key",
        "Keys",
        "Knoll",
        "Knolls",
        "Lake",
        "Lakes",
        "Land",
        "Landing",
        "Lane",
        "Light",
        "Lights",
        "Loaf",
        "Lock",
        "Locks",
        "Locks",
        "Lodge",
        "Lodge",
        "Loop",
        "Mall",
        "Manor",
        "Manors",
        "Meadow",
        "Meadows",
        "Mews",
        "Mill",
        "Mills",
        "Mission",
        "Mission",
        "Motorway",
        "Mount",
        "Mountain",
        "Mountain",
        "Mountains",
        "Mountains",
        "Neck",
        "Orchard",
        "Oval",
        "Overpass",
        "Park",
        "Parks",
        "Parkway",
        "Parkways",
        "Pass",
        "Passage",
        "Path",
        "Pike",
        "Pine",
        "Pines",
        "Place",
        "Plain",
        "Plains",
        "Plains",
        "Plaza",
        "Plaza",
        "Point",
        "Points",
        "Port",
        "Port",
        "Ports",
        "Ports",
        "Prairie",
        "Prairie",
        "Radial",
        "Ramp",
        "Ranch",
        "Rapid",
        "Rapids",
        "Rest",
        "Ridge",
        "Ridges",
        "River",
        "Road",
        "Road",
        "Roads",
        "Roads",
        "Route",
        "Row",
        "Rue",
        "Run",
        "Shoal",
        "Shoals",
        "Shore",
        "Shores",
        "Skyway",
        "Spring",
        "Springs",
        "Springs",
        "Spur",
        "Spurs",
        "Square",
        "Square",
        "Squares",
        "Squares",
        "Station",
        "Station",
        "Stravenue",
        "Stravenue",
        "Stream",
        "Stream",
        "Street",
        "Street",
        "Streets",
        "Summit",
        "Summit",
        "Terrace",
        "Throughway",
        "Trace",
        "Track",
        "Trafficway",
        "Trail",
        "Trail",
        "Tunnel",
        "Tunnel",
        "Turnpike",
        "Turnpike",
        "Underpass",
        "Union",
        "Unions",
        "Valley",
        "Valleys",
        "Via",
        "Viaduct",
        "View",
        "Views",
        "Village",
        "Village",
        "Villages",
        "Ville",
        "Vista",
        "Vista",
        "Walk",
        "Walks",
        "Wall",
        "Way",
        "Ways",
        "Well",
        "Wells",
    )

    POSTAL_ZONES = (
        "AB",
        "AL",
        "B",
        "BA",
        "BB",
        "BD",
        "BH",
        "BL",
        "BN",
        "BR",
        "BS",
        "BT",
        "CA",
        "CB",
        "CF",
        "CH",
        "CM",
        "CO",
        "CR",
        "CT",
        "CV",
        "CW",
        "DA",
        "DD",
        "DE",
        "DG",
        "DH",
        "DL",
        "DN",
        "DT",
        "DY",
        "E",
        "EC",
        "EH",
        "EN",
        "EX",
        "FK",
        "FY",
        "G",
        "GL",
        "GY",
        "GU",
        "HA",
        "HD",
        "HG",
        "HP",
        "HR",
        "HS",
        "HU",
        "HX",
        "IG",
        "IM",
        "IP",
        "IV",
        "JE",
        "KA",
        "KT",
        "KW",
        "KY",
        "L",
        "LA",
        "LD",
        "LE",
        "LL",
        "LN",
        "LS",
        "LU",
        "M",
        "ME",
        "MK",
        "ML",
        "N",
        "NE",
        "NG",
        "NN",
        "NP",
        "NR",
        "NW",
        "OL",
        "OX",
        "PA",
        "PE",
        "PH",
        "PL",
        "PO",
        "PR",
        "RG",
        "RH",
        "RM",
        "S",
        "SA",
        "SE",
        "SG",
        "SK",
        "SL",
        "SM",
        "SN",
        "SO",
        "SP",
        "SR",
        "SS",
        "ST",
        "SW",
        "SY",
        "TA",
        "TD",
        "TF",
        "TN",
        "TQ",
        "TR",
        "TS",
        "TW",
        "UB",
        "W",
        "WA",
        "WC",
        "WD",
        "WF",
        "WN",
        "WR",
        "WS",
        "WV",
        "YO",
        "ZE",
    )

    POSTAL_ZONES_ONE_CHAR = [zone for zone in POSTAL_ZONES if len(zone) == 1]
    POSTAL_ZONES_TWO_CHARS = [zone for zone in POSTAL_ZONES if len(zone) == 2]

    postcode_formats = (
        "AN NEE",
        "ANN NEE",
        "PN NEE",
        "PNN NEE",
        "ANC NEE",
        "PND NEE",
    )

    _postcode_sets = OrderedDict(
        (
            (" ", " "),
            ("N", [str(i) for i in range(0, 10)]),
            ("A", POSTAL_ZONES_ONE_CHAR),
            ("B", "ABCDEFGHKLMNOPQRSTUVWXY"),
            ("C", "ABCDEFGHJKSTUW"),
            ("D", "ABEHMNPRVWXY"),
            ("E", "ABDEFGHJLNPQRSTUWXYZ"),
            ("P", POSTAL_ZONES_TWO_CHARS),
        )
    )

    city_formats = (
        "{{city_prefix}} {{first_name}}{{city_suffix}}",
        "{{city_prefix}} {{first_name}}",
        "{{first_name}}{{city_suffix}}",
        "{{last_name}}{{city_suffix}}",
    )
    street_name_formats = (
        "{{first_name}} {{street_suffix}}",
        "{{last_name}} {{street_suffix}}",
    )
    street_address_formats = (
        "{{building_number}} {{street_name}}",
        "{{secondary_address}}\n{{street_name}}",
    )
    address_formats = ("{{street_address}}\n{{city}}\n{{postcode}}",)
    secondary_address_formats = (
        "Flat #",
        "Flat ##",
        "Flat ##?",
        "Studio #",
        "Studio ##",
        "Studio ##?",
    )

    def postcode(self) -> str:
        """
        See
        http://web.archive.org/web/20090930140939/http://www.govtalk.gov.uk/gdsc/html/noframes/PostCode-2-1-Release.htm
        """
        postcode = ""
        pattern: str = self.random_element(self.postcode_formats)
        for placeholder in pattern:
            postcode += self.random_element(self._postcode_sets[placeholder])
        return postcode

    def city_prefix(self) -> str:
        return self.random_element(self.city_prefixes)

    def secondary_address(self) -> str:
        return self.bothify(self.random_element(self.secondary_address_formats))

    def administrative_unit(self) -> str:
        return self.random_element(self.counties)

    county = administrative_unit



from .. import Provider as BaseProvider


class Provider(BaseProvider):
    """Job provider for mk_MK locale (Macedonian)."""

    jobs = (
        "Агроном",
        "Адвокат",
        "Администратор",
        "Аналитичар",
        "Архитект",
        "Биолог",
        "Биохемичар",
        "Бравар",
        "Ветеринар",
        "Водоинсталатер",
        "Возач",
        "Геодет",
        "Геолог",
        "Градежен инженер",
        "Графички дизајнер",
        "Директор",
        "Доктор по медицина",
        "Економист",
        "Електричар",
        "Електроинженер",
        "Електротехничар",
        "Журналист",
        "Ѕидар",
        "Забар",
        "Инспектор",
        "Информатичар",
        "Истражувач",
        "Книговодител",
        "Козметичар",
        "Конструктор",
        "Кувар",
        "Лекар",
        "Логопед",
        "Мајстор",
        "Машински инженер",
        "Менаџер",
        "Механичар",
        "Молер",
        "Наставник",
        "Нотар",
        "Нутриционист",
        "Пекар",
        "Пилот",
        "Планер",
        "Полицаец",
        "Правник",
        "Програмер",
        "Психолог",
        "Референт",
        "Сoобраќаен инженер",
        "Секретар",
        "Сметководител",
        "Социјален работник",
        "Столар",
        "Стоматолог",
        "Судија",
        "Технолог",
        "Технолошки инженер",
        "Трговец",
        "Учител",
        "Фармацевт",
        "Физиотерапевт",
        "Финансиски аналитичар",
        "Хемичар",
        "Шумар",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    formats_female = (
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}",
        "{{first_name_female}} {{last_name}}-{{last_name}}",
    )

    formats_male = (
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}",
        "{{first_name_male}} {{last_name}}-{{last_name}}",
    )

    formats = formats_male + formats_female

    # source:
    # http://www.bfs.admin.ch/bfs/portal/fr/index/news/publikationen.html?publicationID=6704
    first_names_male = [
        "Alain",
        "Albert",
        "Alexandre",
        "André",
        "Antonio",
        "Arthur",
        "Bernard",
        "Bruno",
        "Charles",
        "Christian",
        "Christophe",
        "Claude",
        "Daniel",
        "David",
        "Eric",
        "Ethan",
        "Florian",
        "François",
        "Frédéric",
        "Gabriel",
        "Georges",
        "Gilbert",
        "Guillaume",
        "Gérard",
        "Henri",
        "Hugo",
        "Jacques",
        "Jean",
        "Jean-Claude",
        "Jean-Pierre",
        "Jonathan",
        "José",
        "Julien",
        "Kevin",
        "Laurent",
        "Louis",
        "Loïc",
        "Luca",
        "Lucas",
        "Léo",
        "Manuel",
        "Marcel",
        "Mathieu",
        "Matteo",
        "Maurice",
        "Maxime",
        "Michael",
        "Michel",
        "Nathan",
        "Nicolas",
        "Noah",
        "Nolan",
        "Olivier",
        "Pascal",
        "Patrick",
        "Paul",
        "Philippe",
        "Pierre",
        "Raymond",
        "René",
        "Robert",
        "Roger",
        "Roland",
        "Romain",
        "Samuel",
        "Stéphane",
        "Sébastien",
        "Thierry",
        "Thomas",
        "Théo",
        "Vincent",
    ]

    first_names_female = [
        "Alice",
        "Alicia",
        "Ana",
        "Anna",
        "Anne",
        "Aurélie",
        "Camille",
        "Caroline",
        "Catherine",
        "Chantal",
        "Charlotte",
        "Chloé",
        "Christiane",
        "Christine",
        "Clara",
        "Claudine",
        "Corinne",
        "Céline",
        "Danielle",
        "Denise",
        "Eliane",
        "Elisa",
        "Elisabeth",
        "Elodie",
        "Emilie",
        "Emma",
        "Eva",
        "Fabienne",
        "Françoise",
        "Georgette",
        "Germaine",
        "Hélène",
        "Isabelle",
        "Jacqueline",
        "Jeanne",
        "Jessica",
        "Josiane",
        "Julie",
        "Laetitia",
        "Lara",
        "Laura",
        "Laurence",
        "Liliane",
        "Lisa",
        "Lucie",
        "Léa",
        "Madeleine",
        "Manon",
        "Marcelle",
        "Marguerite",
        "Maria",
        "Marianne",
        "Marie",
        "Mathilde",
        "Monique",
        "Mélanie",
        "Nathalie",
        "Nelly",
        "Nicole",
        "Odette",
        "Patricia",
        "Sandra",
        "Sandrine",
        "Sara",
        "Sarah",
        "Simone",
        "Sophie",
        "Stéphanie",
        "Suzanne",
        "Sylvie",
        "Thérèse",
        "Valérie",
        "Vanessa",
        "Véronique",
        "Yvette",
        "Yvonne",
        "Zoé",
    ]

    first_names = first_names_male + first_names_female

    # source = http://kunden.eye.ch/swissgen/rsr/index.html
    last_names = [
        "Aebi",
        "Aeby",
        "Alber",
        "Babey",
        "Badan",
        "Badel",
        "Bahon",
        "Balmat",
        "Barbey",
        "Barillon",
        "Barman",
        "Bavaud",
        "Beguin",
        "Berberat",
        "Bernasconi",
        "Besançon",
        "Besençon",
        "Besse",
        "Beuchat",
        "Beuret",
        "Beurret",
        "Blanc",
        "Bochud",
        "Boechat",
        "Boichat",
        "Boillat",
        "Bonvin",
        "Bonvini",
        "Botteron",
        "Bourquard",
        "Bourquin",
        "Bouvier",
        "Bovet",
        "Brahier",
        "Brandt",
        "Broquet",
        "Bugnon",
        "Bujard",
        "Béguelin",
        "Candaux",
        "Carraud",
        "Carraux",
        "Carron",
        "Cattin",
        "Chappuis",
        "Chapuis",
        "Charpié",
        "Chatriand",
        "Chatriant",
        "Chaudet",
        "Chenaux",
        "Chevalley",
        "Chevrolet",
        "Chopard",
        "Coigny",
        "Comman",
        "Comment",
        "Comte",
        "Conrad",
        "Corbat",
        "Corboz",
        "Cornut",
        "Cornuz",
        "Corpataux",
        "Cosandey",
        "Cosendey",
        "Cossy",
        "Courvoisier",
        "Cousin",
        "Cretton",
        "Crevoisier",
        "Crivelli",
        "Curdy",
        "de Dardel",
        "Deladoëy",
        "Delèze",
        "Deshusses",
        "Diesbach",
        "Droz",
        "Dubey",
        "Duroux",
        "Duvanel",
        "Délèze",
        "Evéquoz",
        "Fonjallaz",
        "Francillon",
        "Galland",
        "Georges",
        "Gillièron",
        "Gilliéron",
        "Godet",
        "Grand",
        "Grojean",
        "Grosjean",
        "Gubéran",
        "Humbert",
        "Isella",
        "Jacot-Descombes",
        "Jacot-Guillarmod",
        "Joly",
        "Jomini",
        "Joye",
        "Julliard",
        "Maire",
        "Marti",
        "Martin",
        "Marty",
        "Masseron",
        "Matile",
        "Mayor",
        "Menthonnex",
        "Mercier",
        "Meyer",
        "Monnard",
        "Monnet",
        "Monnet",
        "Monney",
        "Montandon",
        "Morand",
        "Morard",
        "Mottet",
        "Mottiez",
        "Muriset",
        "Musy",
        "Müller",
        "Niquille",
        "Nusslé",
        "Nüsslin",
        "Paccot",
        "Pachoud",
        "Paschoud",
        "Pasquier",
        "Peitrequin",
        "Pellet",
        "Piccand",
        "Polla",
        "Privet",
        "Quartier",
        "Rapin",
        "Rappaz",
        "Rapraz",
        "Rey",
        "Robadey",
        "Robert",
        "Romanens",
        "Rosselat",
        "Rosselet",
        "Rossellat",
        "Sandoz",
        "Sansonnens",
        "Saudan",
        "Thorens",
        "Théraulaz",
        "Tinguely",
        "Treboux",
        "Uldry",
        "Vallélian",
        "Vermeil",
        "Vienne",
        "Vonlanthen",
        "Vuille",
        "Wicht",
    ]



from .. import Provider as DateTimeProvider


class Provider(DateTimeProvider):
    """Date/time provider for mk_MK locale (Macedonian)."""

    # Days of week in Macedonian
    DAY_NAMES = {
        "0": "Понеделник",
        "1": "Вторник",
        "2": "Среда",
        "3": "Четврток",
        "4": "Петок",
        "5": "Сабота",
        "6": "Недела",
    }

    # Months in Macedonian
    MONTH_NAMES = {
        "01": "Јануари",
        "02": "Февруари",
        "03": "Март",
        "04": "Април",
        "05": "Мај",
        "06": "Јуни",
        "07": "Јули",
        "08": "Август",
        "09": "Септември",
        "10": "Октомври",
        "11": "Ноември",
        "12": "Декември",
    }

    def day_of_week(self) -> str:
        return self.DAY_NAMES[self.date("%w")]

    def month_name(self) -> str:
        return self.MONTH_NAMES[self.month()]



from collections import OrderedDict
from typing import Dict, List, Optional

from ...typing import CardType, CreditCard, DateParseType
from .. import BaseProvider

localized = True


class Provider(BaseProvider):
    """Implement default credit card provider for Faker.

    For all methods that take ``card_type`` as an argument, a random card type
    will be used if the supplied value is ``None``. The list of valid card types
    includes ``'amex'``, ``'diners'``, ``'discover'``, ``'jcb'``, ``'jcb15'``,
    ``'jcb16'``, ``'maestro'``, ``'mastercard'``, ``'visa'``, ``'visa13'``,
    ``'visa16'``, and ``'visa19'``.

    Sources:

    - https://en.wikipedia.org/wiki/Payment_card_number#Issuer_identification_number_.28IIN.29
    - https://www.regular-expressions.info/creditcard.html
    - https://creditcardjs.com/credit-card-type-detection
    """

    prefix_maestro: List[str] = [
        "5018",
        "5020",
        "5038",
        "56##",
        "57##",
        "58##",
        "6304",
        "6759",
        "6761",
        "6762",
        "6763",
        "0604",
        "6390",
    ]
    prefix_mastercard: List[str] = [
        "51",
        "52",
        "53",
        "54",
        "55",
        "222%",
        "223",
        "224",
        "225",
        "226",
        "227",
        "228",
        "229",
        "23",
        "24",
        "25",
        "26",
        "270",
        "271",
        "2720",
    ]
    prefix_visa: List[str] = ["4"]
    prefix_amex: List[str] = ["34", "37"]
    prefix_discover: List[str] = ["6011", "65"]
    prefix_diners: List[str] = ["300", "301", "302", "303", "304", "305", "36", "38"]
    prefix_jcb16: List[str] = ["35"]
    prefix_jcb15: List[str] = ["2131", "1800"]

    credit_card_types: Dict[str, CreditCard] = OrderedDict(
        (
            ("maestro", CreditCard("Maestro", prefix_maestro, 12, security_code="CVV")),
            (
                "mastercard",
                CreditCard("Mastercard", prefix_mastercard, 16, security_code="CVV"),
            ),
            ("visa16", CreditCard("VISA 16 digit", prefix_visa)),
            ("visa13", CreditCard("VISA 13 digit", prefix_visa, 13)),
            ("visa19", CreditCard("VISA 19 digit", prefix_visa, 19)),
            (
                "amex",
                CreditCard(
                    "American Express",
                    prefix_amex,
                    15,
                    security_code="CID",
                    security_code_length=4,
                ),
            ),
            ("discover", CreditCard("Discover", prefix_discover)),
            ("diners", CreditCard("Diners Club / Carte Blanche", prefix_diners, 14)),
            ("jcb15", CreditCard("JCB 15 digit", prefix_jcb15, 15)),
            ("jcb16", CreditCard("JCB 16 digit", prefix_jcb16)),
        )
    )
    credit_card_types["visa"] = credit_card_types["visa16"]
    credit_card_types["jcb"] = credit_card_types["jcb16"]

    luhn_lookup = {
        "0": 0,
        "1": 2,
        "2": 4,
        "3": 6,
        "4": 8,
        "5": 1,
        "6": 3,
        "7": 5,
        "8": 7,
        "9": 9,
    }

    def credit_card_provider(self, card_type: Optional[CardType] = None) -> str:
        """Generate a credit card provider name."""
        if card_type is None:
            card_type = self.random_element(self.credit_card_types.keys())  # type: ignore[assignment]
        return self._credit_card_type(card_type).name

    def credit_card_number(self, card_type: Optional[CardType] = None) -> str:
        """Generate a valid credit card number."""
        card = self._credit_card_type(card_type)
        prefix: str = self.random_element(card.prefixes)
        number = self._generate_number(self.numerify(prefix), card.length)
        return number

    def credit_card_expire(
        self,
        start: DateParseType = "now",
        end: DateParseType = "+10y",
        date_format: str = "%m/%y",
    ) -> str:
        """Generate a credit card expiry date.

        This method uses |date_time_between| under the hood to generate the
        expiry date, so the ``start`` and ``end`` arguments work in the same way
        here as it would in that method. For the actual formatting of the expiry
        date, |strftime| is used and ``date_format`` is simply passed
        to that method.
        """
        expire_date = self.generator.date_time_between(start, end)
        return expire_date.strftime(date_format)

    def credit_card_full(self, card_type: Optional[CardType] = None) -> str:
        """Generate a set of credit card details."""
        card = self._credit_card_type(card_type)

        tpl = "{provider}\n{owner}\n{number} {expire_date}\n{security}: {security_nb}\n"

        tpl = tpl.format(
            provider=card.name,
            owner=self.generator.parse("{{first_name}} {{last_name}}"),
            number=self.credit_card_number(card),
            expire_date=self.credit_card_expire(),
            security=card.security_code,
            security_nb=self.credit_card_security_code(card),
        )

        return self.generator.parse(tpl)

    def credit_card_security_code(self, card_type: Optional[CardType] = None) -> str:
        """Generate a credit card security code."""
        sec_len = self._credit_card_type(card_type).security_code_length
        return self.numerify("#" * sec_len)

    def _credit_card_type(self, card_type: Optional[CardType] = None) -> CreditCard:
        """Generate a random CreditCard instance of the specified card type."""
        if card_type is None:
            card_type = self.random_element(self.credit_card_types.keys())  # type: ignore[assignment]
        elif isinstance(card_type, CreditCard):
            return card_type
        return self.credit_card_types[card_type]  # type: ignore[index]

    def _generate_number(self, prefix: str, length: int) -> str:
        """Generate a credit card number.

        The ``prefix`` argument is the start of the CC number as a string which
         may contain any number of digits. The ``length`` argument is the length
         of the CC number to generate which is typically 13 or 16.
        """
        number = prefix
        # Generate random char digits
        number += "#" * (length - len(prefix) - 1)
        number = self.numerify(number)
        reverse = number[::-1]
        # Calculate sum
        tot = 0
        pos = 0
        while pos < length - 1:
            tot += Provider.luhn_lookup[reverse[pos]]
            if pos != (length - 2):
                tot += int(reverse[pos + 1])
            pos += 2
        # Calculate check digit
        check_digit = (10 - (tot % 10)) % 10
        number += str(check_digit)
        return number



from .. import Provider as BankProvider


class Provider(BankProvider):
    """Implement bank provider for ``en_GB`` locale.

    Banks list sourced from:
    https://en.wikipedia.org/wiki/List_of_banks_in_the_United_Kingdom
    https://en.wikipedia.org/wiki/Banking_in_the_United_Kingdom
    Last checked: 2026-04-10
    """

    bban_format = "????##############"
    country_code = "GB"
    banks = (
        "Al Rayan Bank",
        "Aldermore Bank",
        "Atom Bank",
        "Bank of Ireland UK",
        "Bank of London and The Middle East",
        "Bank of Scotland",
        "Barclays Bank",
        "Chase UK",
        "Clydesdale Bank",
        "Co-operative Bank",
        "Gatehouse Bank",
        "Halifax",
        "Handelsbanken",
        "HSBC UK",
        "Investec Bank",
        "Kroo Bank",
        "Lloyds Bank",
        "Metro Bank",
        "Monzo Bank",
        "Nationwide Building Society",
        "NatWest",
        "OakNorth Bank",
        "Revolut Bank",
        "Royal Bank of Scotland",
        "Santander UK",
        "Shawbrook Bank",
        "Starling Bank",
        "Tandem Bank",
        "TSB Bank",
        "Ulster Bank",
        "Virgin Money",
        "Yorkshire Bank",
        "Zempler Bank",
        "Zopa Bank",
    )



from .. import Provider as PersonProvider


class Provider(PersonProvider):
    """Person provider for mk_MK locale (Macedonian).

    Sources:

    - https://www.stat.gov.mk/Iminja_Preziminja.aspx (State Statistical Office of North Macedonia)
    - https://www.behindthename.com/names/usage/macedonian
    """

    first_names_male = (
        "Александар",
        "Алексеј",
        "Андреј",
        "Антон",
        "Арсен",
        "Атанас",
        "Благој",
        "Бојан",
        "Борис",
        "Борче",
        "Бранко",
        "Васил",
        "Видое",
        "Влатко",
        "Гоце",
        "Горан",
        "Дамјан",
        "Даниел",
        "Дарко",
        "Дејан",
        "Димитар",
        "Доне",
        "Драган",
        "Душан",
        "Ѓорги",
        "Ѓорче",
        "Емил",
        "Жарко",
        "Зоран",
        "Иван",
        "Игор",
        "Илија",
        "Јане",
        "Јован",
        "Јовче",
        "Кирил",
        "Коста",
        "Крсте",
        "Кузман",
        "Лазар",
        "Лука",
        "Марјан",
        "Марко",
        "Методи",
        "Михаил",
        "Митко",
        "Момчило",
        "Наум",
        "Никола",
        "Огнен",
        "Павле",
        "Петар",
        "Петре",
        "Ристо",
        "Сашо",
        "Симон",
        "Слободан",
        "Стефан",
        "Стојан",
        "Тихомир",
        "Тодор",
        "Томе",
        "Трајан",
        "Трајче",
        "Филип",
        "Христо",
        "Цветан",
        "Чедо",
    )

    first_names_female = (
        "Александра",
        "Ана",
        "Ангела",
        "Анета",
        "Анита",
        "Билјана",
        "Благица",
        "Валентина",
        "Василка",
        "Васка",
        "Весна",
        "Виолета",
        "Гордана",
        "Даница",
        "Данче",
        "Дарија",
        "Димка",
        "Драгана",
        "Елена",
        "Елизабета",
        "Емилија",
        "Иванка",
        "Јасмина",
        "Јована",
        "Катерина",
        "Кристина",
        "Лилјана",
        "Лина",
        "Магдалена",
        "Маја",
        "Марија",
        "Марина",
        "Мартина",
        "Милена",
        "Мирјана",
        "Надица",
        "Наташа",
        "Невена",
        "Нина",
        "Оливера",
        "Роза",
        "Сања",
        "Сара",
        "Симона",
        "Снежана",
        "Соња",
        "Софија",
        "Стефанија",
        "Сузана",
        "Тања",
        "Теодора",
        "Тијана",
        "Цвета",
        "Цветанка",
    )

    last_names_male = (
        "Алексовски",
        "Андоновски",
        "Апостолов",
        "Арсовски",
        "Атанасов",
        "Бојаџиев",
        "Василев",
        "Велјанов",
        "Велков",
        "Гавриловски",
        "Георгиев",
        "Глигоров",
        "Ѓоргиев",
        "Ѓорѓиевски",
        "Гроздановски",
        "Дамјановски",
        "Дешановски",
        "Димитриев",
        "Димовски",
        "Донев",
        "Дурловски",
        "Заев",
        "Зафировски",
        "Иванов",
        "Илиев",
        "Јанев",
        "Јовановски",
        "Јовчев",
        "Климовски",
        "Коцев",
        "Крстев",
        "Крстески",
        "Лазаревски",
        "Лазов",
        "Марковски",
        "Мартиновски",
        "Матевски",
        "Митревски",
        "Михајловски",
        "Младеновски",
        "Мојсов",
        "Мусијановски",
        "Наумовски",
        "Николов",
        "Николовски",
        "Пановски",
        "Петковски",
        "Петровски",
        "Поповски",
        "Ристовски",
        "Симоновски",
        "Спасов",
        "Спасовски",
        "Стефановски",
        "Стојановски",
        "Тодоровски",
        "Трајковски",
        "Христов",
        "Цветковски",
        "Чулев",
        "Шакири",
    )

    last_names_female = (
        "Алексовска",
        "Андоновска",
        "Апостолова",
        "Арсовска",
        "Атанасова",
        "Бојаџиева",
        "Василева",
        "Велјанова",
        "Велкова",
        "Гавриловска",
        "Георгиева",
        "Глигорова",
        "Ѓоргиева",
        "Ѓорѓиевска",
        "Гроздановска",
        "Дамјановска",
        "Дешановска",
        "Димитриева",
        "Димовска",
        "Донева",
        "Дурловска",
        "Заева",
        "Зафировска",
        "Иванова",
        "Илиева",
        "Јанева",
        "Јовановска",
        "Јовчева",
        "Климовска",
        "Коцева",
        "Крстева",
        "Крстеска",
        "Лазаревска",
        "Лазова",
        "Марковска",
        "Мартиновска",
        "Матевска",
        "Митревска",
        "Михајловска",
        "Младеновска",
        "Мојсова",
        "Николова",
        "Николовска",
        "Пановска",
        "Петковска",
        "Петровска",
        "Поповска",
        "Ристовска",
        "Симоновска",
        "Спасова",
        "Спасовска",
        "Стефановска",
        "Стојановска",
        "Тодоровска",
        "Трајковска",
        "Христова",
        "Цветковска",
        "Чулева",
        "Шакири",
    )

    prefixes_male = ("Г-н", "Др.", "Проф.")
    prefixes_female = ("Г-ѓа", "Г-ца", "Др.", "Проф.")

    formats_male = (
        "{{first_name_male}} {{last_name_male}}",
        "{{prefix_male}} {{first_name_male}} {{last_name_male}}",
    )

    formats_female = (
        "{{first_name_female}} {{last_name_female}}",
        "{{prefix_female}} {{first_name_female}} {{last_name_female}}",
    )

    formats = formats_male + formats_female
    first_names = first_names_male + first_names_female
    last_names = last_names_male + last_names_female



from typing import Tuple, Union

from .. import Provider as BaseProvider


def calculate_checksum(ssn_without_checksum: Union[int, str]) -> int:
    # For Corsican birthplaces, the NIR checksum is computed with 2A -> 19 and 2B -> 18.
    normalized_ssn = str(ssn_without_checksum).replace("2A", "19").replace("2B", "18")
    return 97 - (int(normalized_ssn) % 97)


class Provider(BaseProvider):
    """
    A Faker provider for French social security and VAT IDs.
    """

    vat_id_formats = (
        "FR?? #########",
        "FR## #########",
        "FR?# #########",
        "FR#? #########",
    )

    # department id, municipality id, name of department, name of municipality
    # department id + municipality id = INSEE code
    # municipality ids are only unique within a department
    departments_and_municipalities = (
        # France métropolitaine = Mainland France
        ("01", "053", "Ain", "Bourg-en-Bresse"),
        ("02", "408", "Aisne", "Laon"),
        ("03", "190", "Allier", "Moulins"),
        ("04", "070", "Alpes-de-Haute-Provence", "Digne-les-Bains"),
        ("05", "061", "Hautes-Alpes", "Gap"),
        ("06", "088", "Alpes-Maritimes", "Nice"),
        ("07", "186", "Ardèche", "Privas"),
        ("08", "105", "Ardennes", "Charleville-Mézières"),
        ("09", "122", "Ariège", "Foix"),
        ("10", "387", "Aube", "Troyes"),
        ("11", "069", "Aude", "Carcassonne"),
        ("12", "202", "Aveyron", "Rodez"),
        ("13", "055", "Bouches-du-Rhône", "Marseille"),
        ("14", "118", "Calvados", "Caen"),
        ("15", "014", "Cantal", "Aurillac"),
        ("16", "015", "Charente", "Angoulême"),
        ("17", "300", "Charente-Maritime", "La Rochelle"),
        ("18", "033", "Cher", "Bourges"),
        ("19", "272", "Corrèze", "Tulle"),
        ("2A", "004", "Corse-du-Sud", "Ajaccio"),
        ("2B", "033", "Haute-Corse", "Bastia"),
        ("21", "231", "Côte-d'Or", "Dijon"),
        ("22", "278", "Côtes-d'Armor", "Saint-Brieuc"),
        ("23", "096", "Creuse", "Guéret"),
        ("24", "322", "Dordogne", "Périgueux"),
        ("25", "056", "Doubs", "Besançon"),
        ("26", "362", "Drôme", "Valence"),
        ("27", "229", "Eure", "Évreux"),
        ("28", "085", "Eure-et-Loir", "Chartres"),
        ("29", "232", "Finistère", "Quimper"),
        ("30", "189", "Gard", "Nîmes"),
        ("31", "555", "Haute-Garonne", "Toulouse"),
        ("32", "013", "Gers", "Auch"),
        ("33", "063", "Gironde", "Bordeaux"),
        ("34", "172", "Hérault", "Montpellier"),
        ("35", "238", "Ille-et-Vilaine", "Rennes"),
        ("36", "044", "Indre", "Châteauroux"),
        ("37", "261", "Indre-et-Loire", "Tours"),
        ("38", "185", "Isère", "Grenoble"),
        ("39", "300", "Jura", "Lons-le-Saunier"),
        ("40", "192", "Landes", "Mont-de-Marsan"),
        ("41", "018", "Loir-et-Cher", "Blois"),
        ("42", "218", "Loire", "Saint-Étienne"),
        ("43", "157", "Haute-Loire", "Le Puy-en-Velay"),
        ("44", "109", "Loire-Atlantique", "Nantes"),
        ("45", "234", "Loiret", "Orléans"),
        ("46", "042", "Lot", "Cahors"),
        ("47", "001", "Lot-et-Garonne", "Agen"),
        ("48", "095", "Lozère", "Mende"),
        ("49", "007", "Maine-et-Loire", "Angers"),
        ("50", "502", "Manche", "Saint-Lô"),
        ("51", "108", "Marne", "Châlons-en-Champagne"),
        ("52", "121", "Haute-Marne", "Chaumont"),
        ("53", "130", "Mayenne", "Laval"),
        ("54", "395", "Meurthe-et-Moselle", "Nancy"),
        ("55", "029", "Meuse", "Bar-le-Duc"),
        ("56", "260", "Morbihan", "Vannes"),
        ("57", "463", "Moselle", "Metz"),
        ("58", "194", "Nièvre", "Nevers"),
        ("59", "350", "Nord", "Lille"),
        ("60", "057", "Oise", "Beauvais"),
        ("61", "001", "Orne", "Alençon"),
        ("62", "041", "Pas-de-Calais", "Arras"),
        ("63", "113", "Puy-de-Dôme", "Clermont-Ferrand"),
        ("64", "445", "Pyrénées-Atlantiques", "Pau"),
        ("65", "440", "Hautes-Pyrénées", "Tarbes"),
        ("66", "136", "Pyrénées-Orientales", "Perpignan"),
        ("67", "482", "Bas-Rhin", "Strasbourg"),
        ("68", "066", "Haut-Rhin", "Colmar"),
        ("69", "123", "Rhône", "Lyon"),
        ("70", "550", "Haute-Saône", "Vesoul"),
        ("71", "270", "Saône-et-Loire", "Mâcon"),
        ("72", "181", "Sarthe", "Le Mans"),
        ("73", "065", "Savoie", "Chambéry"),
        ("74", "010", "Haute-Savoie", "Annecy"),
        ("75", "056", "Paris", "Paris"),
        ("76", "540", "Seine-Maritime", "Rouen"),
        ("77", "288", "Seine-et-Marne", "Melun"),
        ("78", "646", "Yvelines", "Versailles"),
        ("79", "191", "Deux-Sèvres", "Niort"),
        ("80", "021", "Somme", "Amiens"),
        ("81", "004", "Tarn", "Albi"),
        ("82", "121", "Tarn-et-Garonne", "Montauban"),
        ("83", "137", "Var", "Toulon"),
        ("84", "007", "Vaucluse", "Avignon"),
        ("85", "191", "Vendée", "La Roche-sur-Yon"),
        ("86", "194", "Vienne", "Poitiers"),
        ("87", "085", "Haute-Vienne", "Limoges"),
        ("88", "160", "Vosges", "Épinal"),
        ("89", "024", "Yonne", "Auxerre"),
        ("90", "010", "Territoire de Belfort", "Belfort"),
        ("91", "228", "Essonne", "Évry-Courcouronnes"),
        ("92", "050", "Hauts-de-Seine", "Nanterre"),
        ("93", "008", "Seine-Saint-Denis", "Bobigny"),
        ("94", "028", "Val-de-Marne", "Créteil"),
        ("95", "500", "Val-d'Oise", "Pontoise"),
        # DROM-COM = Overseas France
        ("971", "05", "Guadeloupe", "Basse-Terre"),
        ("972", "09", "Martinique", "Fort-de-France"),
        ("973", "02", "Guyane", "Cayenne"),
        ("974", "11", "La Réunion", "Saint-Denis"),
        ("976", "11", "Mayotte", "Mamoudzou"),
    )

    def ssn(self) -> str:
        """
        Creates a French SSN (numéro de sécurité sociale) with checksum. Can include letters A or B for Corsica.
        https://fr.wikipedia.org/wiki/Num%C3%A9ro_de_s%C3%A9curit%C3%A9_sociale_en_France#Signification_des_chiffres_du_NIR
        https://www.comptavoo.com/Numero-Securite-sociale,348.html
        :return: a French SSN
        """
        gender_id = self.random_int(min=1, max=2)
        year_of_birth = self.random_int(min=0, max=99)
        month_of_birth = self.random_int(min=1, max=12)
        department_and_municipality: Tuple[str, str, str, str] = self.random_element(
            self.departments_and_municipalities,
        )
        code_department = department_and_municipality[0]
        code_municipality = department_and_municipality[1]

        order_number = self.random_int(min=1, max=999)

        ssn_without_checksum = (
            f"{gender_id:01}{year_of_birth:02}{month_of_birth:02}{code_department}{code_municipality}{order_number:03}"
        )
        checksum = calculate_checksum(ssn_without_checksum)

        return f"{ssn_without_checksum}{checksum:02}"

    def vat_id(self) -> str:
        """
        http://ec.europa.eu/taxation_customs/vies/faq.html#item_11
        :return: a random French VAT ID
        """

        return self.bothify(self.random_element(self.vat_id_formats))
